/******************************************************************************
													            Copyright (c) Freescale 2009
File Name    : LCD_MC_MAIN.c

Current Revision :Revision: 2.0 

PURPOSE:      S12HY64 Demo cluster software                 
                                                                          
                                                                       
DESCRIPTION:  This software should be used in conjunction with AXM-0488 
              Dashboard cluster board hardware. It demostrates motor operation
.             including the implementation of SSD via a software driver. LCD operation 
              is also included and the main odometer uses the Freescale EEE driver to 
              remeber the odometer count. The hardware also has a piezo speaker to which 
              a PWM signal is fed to demonstrate the capacity for warning sounds. Furthermore,
              push bottons and LEDs have been used to mimic indicators and warning lights. 
              

                                                                          
UPDATE HISTORY                                                            
REV  AUTHOR    DATE        DESCRIPTION OF CHANGE                          
---  ------    --------    ---------------------                          
1.5  b06320    29/05/09    - initial coding
2.0  b06320    29/09/09    - Motors run smoother and at different speeds to better emulate 
                              a dashboard
	                         - All motors RTZ simultaneously
	                         - Indicator LEDs and hi beam programmed to flash
	                           Better use of the piezoelectric buzzer is made:
                                o	SW5 is a horn (kind of - only so much you can do with a PWM signal!!)
                                o	Indicator clicking sound is added, similar to relay 
                                o	Hazard alarm noise
                           - LCD updates more frequently and quicker, makes demo look better.
                           - SW4 engages/disengages the 'cruise control
 

     *****************************************************************************
     * File created by: Freescale East Kilbride, Scotland MSG Applications Group
     *
     * Freescale Internal Use Only - This demo contains protected patent pending IP
     *  in the SSD driver. As yet no permission has been given to release this software
     *  so until such time the SSD must be deleted prior to customers seeing it 
     *****************************************************************************

                                                                          
******************************************************************************/
/*===========================================================================*/
/* Freescale reserves the right to make changes without further notice to any*/
/* product herein to improve reliability, function, or design. Freescale does*/
/* not assume any  liability arising  out  of the  application or use of any */
/* product,  circuit, or software described herein;  neither  does it convey */
/* any license under its patent rights  nor the  rights of others.  Freescale*/
/* products are not designed, intended,  or authorized for use as components */
/* in  systems  intended  for  surgical  implant  into  the  body, or  other */
/* applications intended to support life, or  for any  other application  in */
/* which the failure of the Freescale product  could create a situation where*/
/* personal injury or death may occur. Should Buyer purchase or use Freescale*/
/* products for any such intended  or unauthorized  application, Buyer shall */
/* indemnify and  hold  Freescale  and its officers, employees, subsidiaries,*/
/* affiliates,  and distributors harmless against all claims costs, damages, */
/* and expenses, and reasonable  attorney  fees arising  out of, directly or */
/* indirectly,  any claim of personal injury  or death  associated with such */
/* unintended or unauthorized use, even if such claim alleges that  Freescale*/
/* was negligent regarding the  design  or manufacture of the part. Freescale*/
/* and the Freescale logo* are registered trademarks of Freescale Ltd.       */
/*****************************************************************************/

//===========================INCLUDE FILES ==========================================================
#include <hidef.h>      /* common defines and macros */
#include <mc9s12xhy256.h>     /* derivative information */

//******************************************************
#include "lcddrv.h"
#include "cpu.h"
#include "Motor_Algo.h"
#include "main.h"
#include "NormalDemo.h"

#include "ee_emulation.h"
#include "SSD_SGF18.h"

#include "GD-5506P_DEMO9S12HY64.h"
#include "GD-5506P_enums.h"

#define D4B3
  
// #define READ_DFLASH32(address)    ((UINT32)(*(VUINT32 *SSD_SGF18_NEAR)(address-DFLASH_PROGRAM_OFFSET)))
#define READ_DFLASH32(address) ((UINT16)(*(VUINT16 *SSD_SGF18_NEAR)(((address) & (DFLASH_PAGE_SIZE - 1)) + DFLASH_PAGE_WINDOW_START)))
 
 UINT16  tripA_value, tripB_value;     // 1M miles requires at least 20 bits: 
 UINT32  odometer_value;                                             // 1M  = 1111 0100 0010 0100 0000  
 UINT16  addr_to_odometer_value,*ptr_to_odometer_value, addr_to_tripA_value,*ptr_to_tripA_value;
extern  count=0;                                                    // 99k = 1111 0100 0010 0011 1111

tU16  temperature_value; // Temperature variable 

/******************************************************************************
Function Name	:	Delay
Engineer		  :	r32151
Date			   :	02/06/00
Parameters		:	unsigned int delayTime
Returns			:	NONE
Notes			   :	Simple software delay dependent on CPU clock frequency and
					   compile strategy 
******************************************************************************/
void MDelay(int delayTime)					
  {
    int x;				 //outer loop counter 
    char y;									 //inner loop counter 

    for (x=0; x<delayTime; x++){for (y=0; y<100; y++){}}
  }
  
void MDelay0(int delayTime)					
  {
    int x=0;				 //outer loop counter 
    char y=0;									 //inner loop counter 

    for (x=0; x<delayTime; x++){for (y=0; y<10; y++){}}
  }
 
/******************************************************************************
Function Name	:	Main
Engineer		  :	b06320
Date			    :	29/05/09
Parameters		:	NONE
Returns			  :	NONE
Notes			    :	Main program specifies local variables, and contains functions to 
                initiate and operate the motors, LCD, EEE driver, PWM sound generation
                
******************************************************************************/
 
void main(void) 
  {
  tU08 temperature_ctr=0, cruise =0;
  tU08 i=0,l=0,V=0;
  byte result, inputcounter, c;
  UINT16 returnCode;
  tU16 dist=1000, j;
  unsigned char  w, x, y, z;	/*Simple loop count variables*/


	DisableInterrupts;
	
//	ECLKCTL_NECLK = 0;				// turn on ECLK output 

	CRG_8MHz();				  	// init clock module 4MHz bus clock
	Configure_Ports();			    // Configure I/O ports - from MC code
  	
  PWM_SOUND();                // activate PWM ch3 for sound generation
  LCD_init();                 // Configure LCD, enables all FP pins & LCD

  LCD_RAM_SET_ALL();          // Set all LCD segments
  LCD_RAM_CLEAR_ALL();        // Clear all LCD segments

  SET_LOGO(ON,ON,ON,ON,ON);   // local_lcdrammode, FSL_logo, HY_64_Part_Num, Jellyfish_Name
  
  //--SET_TEMPSEGS(RAM mode, temp_type, temp_scale, char col4);----------------------
 // SET_TEMPSEGS(SEGONOFF, inside_temp, F,ON);     // Display Temp Segments - Case 1: 
//  SET_TEMPSEGS(outside_temp,F,ON);               // Display Temp Segments - Case 2
 // SET_TEMPSEGS(SEGONOFF, engine_temp, F,ON);     // Display Temp Segments - Case 3
 // SET_TEMPSEGS(SEGONOFF, inside_temp, C,ON);     // Display Temp Segments - Case 4
 // SET_TEMPSEGS(SEGONOFF, outside_temp,C,ON);     // Display Temp Segments - Case 5
  SET_TEMPSEGS(engine_temp, C,ON);     // Display Temp Segments - Case 6
   
  #if(CALLBACK_ENABLE == TRUE)  // If the Macro CALLBACK_ENABLE is TRUE set CallBack 
    CallBack = CallBackFunc;
  #endif
  
  //INT_Init();                  // Enable the MCU interrupt  dont need!!!
  
  //====================== Enable and initalise the EEE Driver=======================================
  returnCode = FSL_InitEeprom(); // initilise EEPROM 
  if (EED_OK != returnCode)
  {
      ErrorTrap();
  }
  
  while(BUSY == EE_Status)              // Wait until InitEeprom is done 
  {
      FSL_Main();
  }
  
  if(FAIL == EE_Status)                 // Check if FSL_Main has completed execution successfully 
  {
      
      ErrorTrap();                      // Check the gEE_Error for the error code */
  }

 //================================================================================================

   EnableInterrupts;                    // enable interrupts
 
   INIT_LCD_NUMS();                     // display LCD content upon startup - S12HY device name

 //=========================== Set SSD Driver Parameter ===========================================
	Init_Timers();                        // Initialize Motor Timers
	Init_Motor_Params();                  // Initialize Motor State Machine

 	
  SweepDelay = 1000;
  SettleDelay = 200;

  mtr0 = &Motor0;
  mtr1 = &Motor1;
  mtr2 = &Motor2;
  mtr3 = &Motor3;

  RTZallmotors();   
  DelayMS(1500);
  
                                        // Motor velocity 
  mtr0->u16_cmdvel = 200;                // MPH
  mtr1->u16_cmdvel = 100;                // FUEL
  mtr3->u16_cmdvel = 150;                // TEMP
  mtr2->u16_cmdvel = 300;                // RPM

  mtr0->u8_vscaler = 150;               // Amplitude of the PWM signal driving motor
  mtr1->u8_vscaler = 150;  
  mtr2->u8_vscaler = 150;  
  mtr3->u8_vscaler = 150;  

//================================================================================================


   
   	
//========================== LOOP ROUTINE FOR LCD AND MOTORS ======================================  
  for(;;) 
   { 
        MDelay (1000);                   // Delay of 1ms
        
			  //========================= 'Relay' Lindicator clicking Sound===========================
				for (y=0x00; y<=0xFE; y++)/*Variable loop to generate up slope of sinusoid*/		
					{					
						PWMPER3 = y;	/*Loads PWM duty register*/
						MDelay0(100);				/*Optional delay. See above*/
			 		}
	  				  	
			  for (y=0xFF; y<=0x01; y--)	/*Variable loop to generate down slope of sinusoid*/	
			  		{
			  		  PWMPER3 = y;	/*Loads PWM duty register*/
			  			MDelay0(100);				/*Optional delay. See above*/
			  		}	
			  		
        //======================================================================================


       // =========================== Horn Beep sound on SW5=====================================
       if (PT1AD_PT1AD7 == 0) 
          {
          	for(w=0; w<=0; w++) 				  // Beep 2 times
	        	{
				      for(x=0xFE; x>=0x01; x--)		// Fade control loop
			        {
			        	PWMPER3 = x;	            // Load PWM period register (PWM period variation)*/
			        	for (y=0; y<=x; y++)		  // Variable loop to generate up slope of sinusoid*/
              	{					
					      	PWMPER3 = y;	          // Loads PWM duty register*/
					      	MDelay0(0);				      // Optional delay. See above*/
				         }	  	
			 	    	for (y=0xFF; y>=x; y--)		  // Variable loop to generate down slope of sinusoid*/
			  		   {
			  		     PWMPER3 = y;	            // Loads PWM duty register*/
			  			   MDelay0(0);				      // Optional delay. See above*/
			  		    }	
				      }
	    		     	MDelay0(10);					      // Time between beeps. O gives continuous tone, higher values give greater times*/
	  		    }		                          
	  		
          }
        //==========================================================================================
    
    PTR_PTR1 ^= 1;				// toggle LTurn signal
    PTR_PTR0 ^= 1;				// toggle RTurn signal
//    PTR_PTR2 ^= 1;				// toggle signal
    PTM_PTM0 ^= 1;				// toggle FUEL signal
    PTM_PTM1 ^= 1;				// toggle FOG signal
    PTM_PTM2 ^= 1;				// toggle TEMP  signal
    PTM_PTM3 ^= 1;				// toggle ENGINE signal
    PT01AD_PT0AD0	^= 1;		// toggle OIL signal
    PT01AD_PT0AD1	^= 1;		// toggle HI signal

	 if(!menu_sw){
	 	PTS_PTS4 = 0;				// MPH pointer off
	 	while(!menu_sw);			// wait
	 	PTS_PTS4 = 1;				// pointer on
	 }

	 if(!select_sw){
	 	PTS_PTS6 = 0;				// FUEL pointer off
	 	while(!select_sw);			// wait
	 	PTS_PTS6 = 1;				// pointer on
	 }

	 if(!left_sw){
	 	PTR_PTR3 = 0;				// TEMP pointer off
	 	while(!left_sw);			// wait
	 	PTR_PTR3 = 1;				// pointer on
	 }

	 if(!right_sw){
	 	PTR_PTR2 = 0;				// RPM pointer off
	 	while(!right_sw);		// wait
	 	PTR_PTR2 = 1;				// pointer on
	 }

    SEG_P3  ^= 1;                          //Jellyfish_Tentacles;    
    
    ssd_driver();					                 // Init SSD motor controller for VID23-05 motor

 	  if (l == 8) l=0; else l++;;            // Update compass "value"
    SET_GEAR(ON,l);                        // Rolls through gear selector
    
    switch(l) 
      {
      case 0:   SET_COMPASS(ON,N);  break; // update compass
      case 1:   SET_COMPASS(ON,NE); break; // update compass
      case 2:   SET_COMPASS(ON,E);  break; // update compass
      case 3:   SET_COMPASS(ON,SE); break; // update compass
      case 4:   SET_COMPASS(ON,S);  break; // update compass
      case 5:   SET_COMPASS(ON,SW); break; // update compass
      case 6:   SET_COMPASS(ON,W);  break; // update compass
      case 7:   SET_COMPASS(ON,NW); break; // update compass
      }

    SET_VOLT_BAR(ON, V);
    SET_TEMP_BAR(ON, (6-V));               // Update Temp Bar Graph w/inverse of Voltage Graph

 	  if (V == 6) V=0; else V++;             // Update Voltage Bar Graph "value"
 	  
 	  if (dist<=500)                         // short routine for altering the LCD display when distance < 500
 	    {
 	      TYRE_CONFIG (ON,ON,ON,ON);         // low tire pressure
 	    }
 	  else
 	    {
 	      TYRE_CONFIG (ON,ON,OFF,OFF);        // normal running cruise engaged
 	    }
 	    
 	  
 	  SET_DIST_TITLE (ON,ON,ON);              // set distance remaining
 	  DIST_REMAIN(ON, dist);                  // set the counter for miles left in the tank.
    dist--; if (dist==0) dist=1000;         // decrement the distance number
 	  
 	  SET_TEMP_DIGITS(ON,temperature_ctr);     // Update temperature digits with value of i
    temperature_ctr++; if (temperature_ctr==255) temperature_ctr=0;
    
          
switch(count)   
    {
      // ODOMETER
      case 0:
                
          SET_ODO_TITLE (ON,ON,OFF,OFF,OFF);   // (ODO, MILES, TRIPA, TRIPB, KM) -->SEGs
          returnCode = FSL_ReadEeprom(DATA_ID_ONE, &addr_to_odometer_value);// Read the previous odometer_value 
          
          if (EED_OK != returnCode) 
             {
               // Then the dataID read is not found 
             } 
          else 
             {
               odometer_value = READ_DFLASH32(addr_to_odometer_value);
             }
                
          SET_ODO_DIGITS(ON,odometer_value);   // Update odometer digits with global odometer value
          odometer_value++; if (odometer_value==1000000) odometer_value=0; // Write the odometer_value to the Emulated EEPROM 
          ptr_to_odometer_value = &odometer_value;
          returnCode = FSL_WriteEeprom(DATA_ID_ONE, ptr_to_odometer_value);
          
          if (EED_OK != returnCode)
             {
               ErrorTrap();
              }
                
          while(BUSY == EE_Status)              // Wait until WriteEeprom is done 
             {
               FSL_Main();
             }
                
          if(FAIL == EE_Status)                // Check if FSL_Main has completed execution successfully 
             {
                    
                 ErrorTrap();                  // Check the gEE_Error for the error code 
              }
                break;
                      
        //TRIP A Meter
        case 1: SET_ODO_TITLE (OFF,ON,ON,OFF,OFF);   // (ODO, MILES, TRIPA, TRIPB, KM) -->SEGs
               
                SET_TRIPA_DIGITS(ON,tripA_value);   // Update tripA digits with global tripA value
                tripA_value++; 
                if (tripA_value==1000000) tripA_value=0;
                
                /*********Must also increment the odometer in the background*************/
                returnCode = FSL_ReadEeprom(DATA_ID_ONE, &addr_to_odometer_value);
                if (EED_OK != returnCode) 
                {
                  // Then the dataID read is not found 
                } 
                else 
                {
                   odometer_value = READ_DFLASH32(addr_to_odometer_value);
                }
                odometer_value++; if (odometer_value==1000000) odometer_value=0; /* Increment the odometer value*/
                
                ptr_to_odometer_value = &odometer_value;    // Write the odometer_value to the Emulated EEPROM */
                returnCode = FSL_WriteEeprom(DATA_ID_ONE, ptr_to_odometer_value);
                if (EED_OK != returnCode)
                {
                    ErrorTrap();
                }
                /* Wait until WriteEeprom is done */
                while(BUSY == EE_Status)
                {
                    FSL_Main();
                }
                
                if(FAIL == EE_Status)             // Check if FSL_Main has completed execution successfully */
                {
                    
                    ErrorTrap();                 // Check the gEE_Error for the error code 
                }
                
                break;
        
        //TRIP B meter
        case 2: SET_ODO_TITLE (OFF,ON,OFF,ON,OFF);   // (ODO, MILES, TRIPA, TRIPB, KM) -->SEGs
                SET_TRIPB_DIGITS(ON,tripB_value);    // Update tripB digits with global odometer value
                tripB_value++; if (tripB_value==1000000) tripB_value=0; 
                
                                /*********Must also increment the odometer in the background*************/
                returnCode = FSL_ReadEeprom(DATA_ID_ONE, &addr_to_odometer_value);
                if (EED_OK != returnCode) 
                {
                  /* Then the dataID read is not found */
                } 
                else 
                {
                   odometer_value = READ_DFLASH32(addr_to_odometer_value);
                }
                odometer_value++; if (odometer_value==1000000) odometer_value=0; // Increment the odometer value
               
                ptr_to_odometer_value = &odometer_value;    // Write the odometer_value to the Emulated EEPROM 
                returnCode = FSL_WriteEeprom(DATA_ID_ONE, ptr_to_odometer_value);
                if (EED_OK != returnCode)
                {
                    ErrorTrap();
                }
                while(BUSY == EE_Status)            // Wait until WriteEeprom is done 
                {
                    FSL_Main();
                }
                
                if(FAIL == EE_Status)               // Check if FSL_Main has completed execution successfully 
                {
                   ErrorTrap();                     // Check the gEE_Error for the error code 
                }
      }

    } // wait forever 
  
  }


