#define  MOTOR_ALGO_C
#include <hidef.h>
#include <mc9s12xhy256.h>     /* derivative information */
#include "Motor_Algo.h"
#include "main.h"
#include "GD-5506P_DEMO9S12HY64.h"



//----------------------------------------------------------------------------------------------------------------------
// BEGIN Callable functions for Motor Movement
//----------------------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------- 
// WaitForRTZdone()
// 
//-------------------------------------------------------------------- 
void WaitForRTZdone(motorparams *mtr)
{
    while (mtr->u8RTZon != RTZdone);
}
//-------------------------------------------------------------------- 


//-------------------------------------------------------------------- 
// WaitUntilStopped()
// 
//-------------------------------------------------------------------- 
void WaitUntilStopped(motorparams *mtr)
{
    while(mtr->u32_cmdpos != mtr->u32_pos);
}
//-------------------------------------------------------------------- 


//-------------------------------------------------------------------- 
// CheckRTZdone()
// 
//-------------------------------------------------------------------- 
u8 CheckRTZdone(motorparams *mtr)
{
    if(mtr->u8RTZon == RTZdone)
        return(TRUE);
    else
        return(FALSE);
}
//-------------------------------------------------------------------- 


//-------------------------------------------------------------------- 
// CheckIfStopped()
// 
//-------------------------------------------------------------------- 
u8 CheckIfStopped(motorparams *mtr) 
{
    if (mtr->u32_cmdpos != mtr->u32_pos)
      return(FALSE);
    else
      return(TRUE);
}
//-------------------------------------------------------------------- 


//-------------------------------------------------------------------- 
// GoToPosition()
// This is assumed to be in degrees for a 2deg / fullcycle motor
//-------------------------------------------------------------------- 
void GoToPosition(motorparams *mtr, u32 position)
{
    
    if(mtr->u8_mtr == DENSOmtr){
      position = position * 2/3;
    }
    mtr->u32_cmdpos = position << 16;
}
//-------------------------------------------------------------------- 


//-------------------------------------------------------------------- 
// GoToFullStepPosition()
//
//-------------------------------------------------------------------- 
void GoToFullStepPosition(motorparams *mtr, u32 position)
{
    mtr->u32_cmdpos = position << 15;
}
//-------------------------------------------------------------------- 

//-------------------------------------------------------------------- 
// RTZmotor()
//
//-------------------------------------------------------------------- 
void RTZallmotors(void)
{
 	usteps_in_cycle = 32;
  detectdelay = DEFAULT_DETECTDELAY;

  GoToFullStepPosition(mtr0,detectdelay); //pull away from stop the amount it takes to detect RTZ accurately
  GoToFullStepPosition(mtr1,detectdelay); //pull away from stop the amount it takes to detect RTZ accurately
  GoToFullStepPosition(mtr2,detectdelay); //pull away from stop the amount it takes to detect RTZ accurately
  GoToFullStepPosition(mtr3,detectdelay); //pull away from stop the amount it takes to detect RTZ accurately

  WaitUntilStopped(mtr0);           
  WaitUntilStopped(mtr1);           
  WaitUntilStopped(mtr2);           
  WaitUntilStopped(mtr3);
             
  DelayMS(200);             // wait for pointers to settle.
    
  RTZspeed = CLOSEPROXSPEED;
  RTZstartspeed = RTZspeed;
  MSthreshold = USTEP_THRESHOLD_L;

  InitRTZ(mtr0, RTZspeed, RTZstartspeed, MSthreshold, detectdelay);
  InitRTZ(mtr1, RTZspeed, RTZstartspeed, MSthreshold, detectdelay);
  InitRTZ(mtr2, RTZspeed, RTZstartspeed, MSthreshold, detectdelay);
  InitRTZ(mtr3, RTZspeed, RTZstartspeed, MSthreshold, detectdelay);

  mtr0->u8RTZon = RTZnewstep;                           // start RTZ function
  mtr1->u8RTZon = RTZnewstep;                           // start RTZ function
  mtr2->u8RTZon = RTZnewstep;                           // start RTZ function
  mtr3->u8RTZon = RTZnewstep;                           // start RTZ function
  
  WaitForRTZdone(mtr0);
  WaitForRTZdone(mtr1);
  WaitForRTZdone(mtr3);
  WaitForRTZdone(mtr2);
}


//-------------------------------------------------------------------- 
// RTZmotor()
//
//-------------------------------------------------------------------- 
void RTZmotor(motorparams *mtr)
{
   	usteps_in_cycle = 32;
  
//    GoToFullStepPosition(mtr,detectdelay); //pull away from stop the amount it takes to detect RTZ accurately
//    WaitUntilStopped(mtr);           
//    DelayMS(200);                           // wait 200ms for pointer to settle
  
    RTZspeed = CLOSEPROXSPEED;
    RTZstartspeed = 60;
    MSthreshold = USTEP_THRESHOLD_L;

    InitRTZ(mtr, RTZspeed, RTZstartspeed, MSthreshold, detectdelay);
    mtr->u8RTZon = RTZnewstep;
}
//-------------------------------------------------------------------- 


//-------------------------------------------------------------------- 
// void DelayMS()
//
//-------------------------------------------------------------------- 
void DelayMS(u16 delay)
{
    u16SysTimer = 0;
    while (u16SysTimer < delay);
}
//-------------------------------------------------------------------- 


//----------------------------------------------------------------------------------------------------------------------
// void Init_Motor_Params(void)
//
//----------------------------------------------------------------------------------------------------------------------
void Init_Motor_Params(void)
{    
  // init all motor variables
    Motor0.u8_vscaler = 255;
    Motor0.u32_pos = 0;
    Motor0.u32_cmdpos = 0;
    Motor0.u8_decayval = DEFAULT_DECAY;
    Motor0.u16_cmdvel = DEFAULT_CMDVEL;
    Motor0.u16_maxvel = MAXVEL;

    Motor1.u8_vscaler = 255;
    Motor1.u32_pos = 0;
    Motor1.u32_cmdpos = 0;
    Motor1.u8_decayval = DEFAULT_DECAY;
    Motor1.u16_cmdvel = DEFAULT_CMDVEL;
    Motor1.u16_maxvel = MAXVEL;

    Motor2.u8_vscaler = 255;
    Motor2.u32_pos = 0;
    Motor2.u32_cmdpos = 0;
    Motor2.u8_decayval = DEFAULT_DECAY;
    Motor2.u16_cmdvel = DEFAULT_CMDVEL;
    Motor2.u16_maxvel = MAXVEL;

    Motor3.u8_vscaler = 255;
    Motor3.u32_pos = 0;
    Motor3.u32_cmdpos = 0;
    Motor3.u8_decayval = DEFAULT_DECAY;
    Motor3.u16_cmdvel = DEFAULT_CMDVEL;
    Motor3.u16_maxvel = MAXVEL;

    Motor0.u8_pwmchan = 0;  
    Motor1.u8_pwmchan = 1;  
    Motor2.u8_pwmchan = 2;  
    Motor3.u8_pwmchan = 3;  
 
    Motor0.u8_mtr = MOTOR;
    Motor1.u8_mtr = MOTOR;
    Motor2.u8_mtr = MOTOR;
    Motor3.u8_mtr = MOTOR;
  
    Motor0.u8_mtrtype = MOTOR_TYPE;
    Motor1.u8_mtrtype = MOTOR_TYPE;
    Motor2.u8_mtrtype = MOTOR_TYPE;
    Motor3.u8_mtrtype = MOTOR_TYPE;
  
    Motor0.u8_stopdirection = M0_STOP_DIRECTION;
    Motor1.u8_stopdirection = M1_STOP_DIRECTION;
    Motor2.u8_stopdirection = M2_STOP_DIRECTION;
    Motor3.u8_stopdirection = M3_STOP_DIRECTION;

    if (Motor0.u8_stopdirection == CCW) {
      Motor0.u8RTZsample_pt0 = CCWSAMPLEPT0;
      Motor0.u8RTZsample_pt1 = CCWSAMPLEPT1;
      Motor0.u8RTZsample_pt2 = CCWSAMPLEPT2;
      Motor0.u8RTZsample_pt3 = CCWSAMPLEPT3;
    } else {
      Motor0.u8RTZsample_pt0 = CWSAMPLEPT0;
      Motor0.u8RTZsample_pt1 = CWSAMPLEPT1;
      Motor0.u8RTZsample_pt2 = CWSAMPLEPT2;
      Motor0.u8RTZsample_pt3 = CWSAMPLEPT3;
    }

    if (Motor1.u8_stopdirection == CCW) {
      Motor1.u8RTZsample_pt0 = CCWSAMPLEPT0;
      Motor1.u8RTZsample_pt1 = CCWSAMPLEPT1;
      Motor1.u8RTZsample_pt2 = CCWSAMPLEPT2;
      Motor1.u8RTZsample_pt3 = CCWSAMPLEPT3;
    } else {
      Motor1.u8RTZsample_pt0 = CWSAMPLEPT0;
      Motor1.u8RTZsample_pt1 = CWSAMPLEPT1;
      Motor1.u8RTZsample_pt2 = CWSAMPLEPT2;
      Motor1.u8RTZsample_pt3 = CWSAMPLEPT3;
    }

    if (Motor2.u8_stopdirection == CCW) {
      Motor2.u8RTZsample_pt0 = CCWSAMPLEPT0;
      Motor2.u8RTZsample_pt1 = CCWSAMPLEPT1;
      Motor2.u8RTZsample_pt2 = CCWSAMPLEPT2;
      Motor2.u8RTZsample_pt3 = CCWSAMPLEPT3;
    } else {
      Motor2.u8RTZsample_pt0 = CWSAMPLEPT0;
      Motor2.u8RTZsample_pt1 = CWSAMPLEPT1;
      Motor2.u8RTZsample_pt2 = CWSAMPLEPT2;
      Motor2.u8RTZsample_pt3 = CWSAMPLEPT3;
    }

    if (Motor3.u8_stopdirection == CCW) {
      Motor3.u8RTZsample_pt0 = CCWSAMPLEPT0;
      Motor3.u8RTZsample_pt1 = CCWSAMPLEPT1;
      Motor3.u8RTZsample_pt2 = CCWSAMPLEPT2;
      Motor3.u8RTZsample_pt3 = CCWSAMPLEPT3;
    } else {
      Motor3.u8RTZsample_pt0 = CWSAMPLEPT0;
      Motor3.u8RTZsample_pt1 = CWSAMPLEPT1;
      Motor3.u8RTZsample_pt2 = CWSAMPLEPT2;
      Motor3.u8RTZsample_pt3 = CWSAMPLEPT3;
    }

  TEST60 = _60degrees;
  TEST90 = _90degrees;
}	  // End Init_Motor_Params()
//----------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------
// End Callable functions for Motor Movement
//----------------------------------------------------------------------------------------------------------------------



//----------------------------------------------------------------------------------------------------------------------
// void Init_Timers(void)
//
// This function initializes the 9S12XHZ's stepper motor PWMs for gauge control and sets up 
// the General Purpose Timer channels (configured as SW output compares) to be used as the time
// schedulers for the motors.  The assignment is below:
//
// OC0 -  Motor0 timing  (note: these can also be reconfigured to IC do flyback measurement)
// OC1 -  Motor1 timing  
// 
//----------------------------------------------------------------------------------------------------------------------
void Init_Timers(void)
{

    TIM0_TIOS  = 0x05;	// set IOC0_0 and IOC0_2 to be output compares.  These will be used to schedule motor movement for M0 and M1 respectively.
	  TIM0_TCTL2 = 0x00;	// make sure IOC0_0 and IOC0_2 are set up for SW compare only - no action on pin.
	  TIM0_TCTL4 = 0xFF;	// set up IOC0_0 - IOC0_3 to capture on either edge.  This could be used for stall detection on M0 and M1

    TIM1_TIOS  = 0x05;	// set IOC1_0 and IOC1_2 to be output compares.  These will be used to schedule motor movement for M0 and M1 respectively.
	  TIM1_TCTL2 = 0x00;	// make sure IOC1_0 and IOC1_2 are set up for SW compare only - no action on pin.
	  TIM1_TCTL4 = 0xFF;	// set up IOC1_0 - IOC1_3 to capture on either edge.  This could be used for stall detection on M0 and M1

    TIM0_TSCR2 = TIMERPRESCALER;	// TIM0 is divide by 2 = 16MHz/2 = 8MHz  --> 
	  TIM1_TSCR2 = TIMERPRESCALER;	// TIM1 is divide by 2 = 16MHz/2 = 8MHz

    TIM0_TSCR1 = 0xa0;  // enable timer module - stop timer in freeze mode - for DEBUG purposes
    TIM1_TSCR1 = 0xa0;  // enable timer module - stop timer in freeze mode - for DEBUG purposes
    
    TIM0_TIE_C0I = 1;   // enable IOC0_0 interrupts - used to schedule M0 movement
    TIM0_TIE_C2I = 1;	  // enable IOC0_2 interrupts - used to schedule M1 movement

    TIM1_TIE_C0I = 1;   // enable IOC1_0 interrupts - used to schedule M2 movement
    TIM1_TIE_C2I = 1;	  // enable IOC1_2 interrupts - used to schedule M3 movement

    TIM0_TIOS |= 0x10;  // set IOC0_4 to be output compare ..... use for 1ms system tick.
    TIM0_TIE_C4I = 1;   // enable IOC0_4 interrupt
  	TIM0_TC4 = TIM0_TCNT + _1ms;  // SET INTERRUPT RATE for General Purpose System Tic

  	TIM0_TC0 = TIM0_TCNT + _1ms;  // SET INTERRUPT RATE
  	TIM0_TC1 = TIM0_TCNT + _1ms;  // SET INTERRUPT RATE

  	TIM1_TC0 = TIM1_TCNT + _1ms;  // SET INTERRUPT RATE
  	TIM1_TC1 = TIM1_TCNT + _1ms;  // SET INTERRUPT RATE


  //----------------------------
  //  Motor PWM setup code   
  //----------------------------

	// set up Motor Control PWMs

	/*
	|---------------------------------------------------------------------|
	|   0   | MCPRE1 | MCPRE0 | MCSWAI |  FAST  |  DITH  |   0   | MCTOIF |
	|---------------------------------------------------------------------|
    
    MCPRE[1:0] -->  00 = module clock is FBUS
    			          01 = module clock is FBUS/2
				            10 = module clock is FBUS/4
				            11 = module clock is FBUS/8
	
	  MCSWAI = 0 -->  1 = motor coils are released in WAIT mode
					          0 = motor coils are still driven during WAIT mode
	  FAST = 0   -->  1 =  7-bit PWM mode
					          0 = 11-bit PWM mode
	  DITH = 0   -->  1 = Dither enabled
					          0 = Dither disabled	
	*/
	
//	MCCTL0 = 0x04;    // module clock = FBUS = 16MHz
	MCCTL0 = 0x24;    // module clock = FBUS/2 = 8MHz
//	MCCTL0 = 0x44;    // module clock = FBUS/4 = 4MHz

	/*
	|---------------------------------------------------------------------|
	|  RECIRC  |   0   |   0   |   0   |   0   |   0   |   0   |  MCTOIE  |
	|---------------------------------------------------------------------|
	
	  RECIRC = 0 -->  0 = Recirc on high side (PWM active low) 
		  		          1 = Recirc on low side (PWM active high) 
	  MCTOIE = 0 -->  0 = Interrupt disabled
		  		          1 = PWM Timer Interrupt enabled	
	*/

	MCCTL1 = 0x00;

	/*
		FBUS  = 16MHz 
    ftc   = FBUS/4 = 4MHz 

		using 8-bits of resolution:
		250ns * 256 = 32us = PWM_PERIOD
	
		PWM Frequency = 1/PWM_PERIOD = 15.625 KHz	 (4MHz Timer clock)
		PWM Frequency = 1/PWM_PERIOD = 31.25 KHz   (8MHz Timer clock)
	*/

	MCPER = 0x0FF;

	/*	 
	|---------------------------------------------------------------------|
	|   MCOM1  |  MCOM0  |  MCAM1  |  MCAM0  |  0  |  0  |  CD1  |  CD0   |
	|---------------------------------------------------------------------|
    
    MCOM[1:0] --> 00 = Half H-Bridge mode - PWM on MnCxM - MnCxP is released
                  01 = Half H-Bridge mode - PWM on MnCxP - MnCxM is released
			        	  10 = Full H-Bridge mode
		        		  11 = Dual Full H-Bridge mode

	  MCAM[1:0] --> 00 = Channel disabled
		        		  01 = left aligned
				          10 = right aligned
	        			  11 = center aligned
	
	  CD[1:0] ----> 00 = 0 timer clocks of delay
		        		  01 = 1 timer clocks of delay
				          10 = 2 timer clocks of delay
				          11 = 3 timer clocks of delay  				  	
	*/

  	MCCC0 = 0xF0;   // Motor 0
  	MCCC1 = 0xF0;
  	MCCC2 = 0xF0;	  // Motor 1
  	MCCC3 = 0xF0;
  	
  	MCCC4 = 0xF0;  	// Motor 2
  	MCCC5 = 0xF0;
  	MCCC6 = 0xF0;	  // Motor 3
  	MCCC7 = 0xF0;

	// Initial Duty cycle

  	MCDC0 = 0;		  // Motor 0
  	MCDC1 = 0;
  	MCDC2 = 0;		  // Motor 1
  	MCDC3 = 0;
  	
  	MCDC4 = 0;		  // Motor 2
  	MCDC5 = 0;
  	MCDC6 = 0;		  // Motor 3
  	MCDC7 = 0;
  	
}
//----------------------------------------------------------------------------------------------------------------------
// end Init_Timers
//----------------------------------------------------------------------------------------------------------------------



//----------------------------------------------------------------------------------------------------------------------
// InitMicroStepRTZ()
// 
// MicrostepRTZ assumes integration over only a fraction of the microstep.  Therefore, a few key variables are input into
// this routine.
//
// integrationtime --> the time in microseconds to integrate
// rtzspeed        --> speed in deg /s  which determines the microstep time.
// rtzstartspeed   --> startspeed in deg /s  which determines the starup step timee
//
// Since the integrator is based off a 4 microsecond time base, the value in the integrationtime variable needs to be 1/4 
// of the desired integration time,
//      (i.e desired integration time = 200us ---> integrationtime variable = 50)
//
// The microstep time and startspeed time is based on a variables assumed to be in microseconds so the
// u16RTZspeed and u16RTZstartspeed need to be converted to microseconds / microstep.
//
//    1          1000000 usecs      1 deg        ( 1000000 / 8 ) (usecs/ustep)(deg/s)
//  --------- * --------------- * ----------  =  ------------------------------------
//  (x deg/s)         s            8 usteps              (x  deg/s)
//
//
//
//
// InitShortHalfStepRTZ()
// 
// ShortHalfStepRTZ assumes integration over only a fraction of the full step.  Therefore, two key variables are input into ths
// routine:
//
// integrationtime --> the time in microseconds to integrate
// rtzspeed --> the speed in deg /s which determines the full step time.
//
// Since the value in the integrationtime variale is 1/4 of the actual integration time, the integrationtime variable
// needs to be divided by 4
//      ( i.e actual integration time = 8ms = 8000usecs --->  integrationtime variable = 2000 )
//
// The fullstep time is determined by rtzspeed and rtzspeed is assumed to be in deg/s.
// This value is converted into the u16RTZspeed value which is assumed to be in milliseconds.
//
// Therefore, to convert rtzspeed from degrees per second to milliseconds per Halfstep,
// we have:
//
//      1         1000 millisecs       1 deg           1000 (milliseconds)(deg)        ( 1000 / 2 ) (deg/Fullstep)
//  ---------  *  --------------- *  -----------  =  -----------------------------  =  ---------------------------
//  (x deg/s)           s            4 HalfSteps     (x deg/s)(seconds)(Fullsteps)              (x deg/s)
//
//
//
//
// InitShortFullStepRTZ()
// 
// ShortFullStepRTZ assumes integration over only a fraction of the full step.  Therefore, two key variables are input into ths
// routine:
//
// integrationtime --> the time in microseconds to integrate
// rtzspeed --> the speed in deg /s which determines the full step time.
//
// Since the value in the integrationtime variale is 1/4 of the actual integration time, the integrationtime variable
// needs to be divided by 4
//      ( i.e actual integration time = 8ms = 8000usecs --->  integrationtime variable = 2000 )
//
// The fullstep time is determined by rtzspeed and rtzspeed is assumed to be in deg/s.
// This value is converted into the u16RTZspeed value which is assumed to be in milliseconds.
//
// Therefore, to convert rtzspeed from degrees per second to milliseconds per fullstep,
// we have:
//
//      1         1000 millisecs       1 deg           1000 (milliseconds)(deg)        ( 1000 / 2 ) (deg/Fullstep)
//  ---------  *  --------------- *  -----------  =  -----------------------------  =  ---------------------------
//  (x deg/s)           s            2 FullSteps     (x deg/s)(seconds)(Fullsteps)              (x deg/s)
//
//----------------------------------------------------------------------------------------------------------------------
void InitRTZ(motorparams *mtr, u16 rtzspeed, u16 rtzstartspeed, s16 threshold, u8 detectdelay)
{

    u16 u16tmp1,u16tmp2;
    

    #define USTEPconvert          2000000
    
    if (rtzspeed > MAXRTZ)
      rtzspeed = MAXRTZ;
    
    

    AlignMotorOnFullStep(mtr);
    mtr->u8RTZmicrostepsincycle  = usteps_in_cycle; //32;

    mtr->u8RTZdetectdelay = detectdelay;
    mtr->u8RTZincrementer = TABLELENGTH / mtr->u8RTZmicrostepsincycle;
    
    u16tmp1 = rtzspeed * mtr->u8RTZmicrostepsincycle;
    mtr->u16RTZspeed = (u16)(USTEPconvert / u16tmp1);     
    
    u16tmp2 = rtzstartspeed * mtr->u8RTZmicrostepsincycle;
    mtr->u16RTZstartspeed = (u16)(USTEPconvert / u16tmp2);     
    
    
    if (rtzspeed <= STARTSTOPFREQ)    
      mtr->u16RTZstartspeed = mtr->u16RTZspeed;
    else
      mtr->u16RTZstartspeed = mtr->u16RTZstartspeed; //0xFFFF

    mtr->s16RTZthreshold = threshold;   

    mtr->u16RTZcount = 0xFFFF;    // use RTZcount as an indication of step during RTZ
    mtr->u16RTZsteptimer = 0;     // use RTZsteptimer to generate step time.
                
//    mtr->u8RTZon = RTZnewstep;                           // start RTZ function

}
//----------------------------------------------------------------------------------------------------------------------
//  end InitRTZ()
//----------------------------------------------------------------------------------------------------------------------



//----------------------------------------------------------------------------------------------------------------------
// Drive_PWMs()
//
// This function drives the motor coils to their correct step.
// It looks up the microstep value from the sin table
// and outputs it to the Stepper Motor PWMs.
//
// Note: this will generate a 60 degree or 90 degree coil angle 
//       based on the motor type
//        0 = 90 degree coil angle
//        1 = 60 degree coil angle
//----------------------------------------------------------------------------------------------------------------------
void Drive_PWMs(motorparams *mtr)
{

	u16  pwm_val;
	u8   tbl_index;
	u8   vscaler;

	MCCTL0_MCTOIF = 1;             // clear PWM overflow - errata fix 

    
    tbl_index = mtr->u8_step;
    vscaler = mtr->u8_vscaler;
    
//  ** Set Coil2 (SIN coil) **

	pwm_val = SIN_tbl[tbl_index];                     // Get Coil voltage
	pwm_val = ( (vscaler * pwm_val) + 0x0080 )>>8;	  // Scale it if needed

	if (tbl_index < TABLELENGTH/2)
		{ pwm_val &= 0x7FFF;}	// Set the direction of current to be +
	else							
		{ pwm_val |= 0x8000;}	// Set the direction of current to be -
									
	while (MCCTL0_MCTOIF == 0);    // wait for PWM to overflow - errata fix 
	
  if (mtr->u8_pwmchan == 0)
    MOTOR_0_COIL2 = pwm_val;  // Motor 0
  else if (mtr->u8_pwmchan == 1)
  	MOTOR_1_COIL2 = pwm_val;	// Motor 1 
  else if (mtr->u8_pwmchan == 2)
  	MOTOR_2_COIL2 = pwm_val;  // Motor 2
  else
  	MOTOR_3_COIL2 = pwm_val;  // Motor 3


// ** update the table index for the second coil

  if (mtr->u8_mtrtype == _60degType)
    {tbl_index += TEST60;}//_60degrees;}       // Coil angle = 60 degrees = 360/6 --> TABLELENGTH/6
  else
    {tbl_index += TEST90;}//_90degrees;}       // Coil angle = 90 degrees = 360/4 --> TABLELENGTH/4 
  
 tbl_index &= TABLEMASK;             // Check for overflow and loop back around
                                    // NOTE: THIS WORKS ONLY IF TABLELENGTH IS POWER OF 2
// **         


//  ** Set Coil1 (COS coil) **

	pwm_val = SIN_tbl[tbl_index];                     // Get Coil voltage
	pwm_val = ( (vscaler * pwm_val) + 0x0080 )>>8;	  // Scale it if needed

	if ( tbl_index < TABLELENGTH/2)
		{ pwm_val &= 0x7FFF;}	// Set the direction of current to be +
	else							
		{ pwm_val |= 0x8000;}	// Set the direction of current to be -
									
  if (mtr->u8_pwmchan == 0)
    MOTOR_0_COIL1 = pwm_val;	// Motor 0
  else if (mtr->u8_pwmchan == 1)
  	MOTOR_1_COIL1 = pwm_val;	// Motor 1
	else if (mtr->u8_pwmchan == 2)
		MOTOR_2_COIL1 = pwm_val;  // Motor 2
	else
		MOTOR_3_COIL1 = pwm_val;  // Motor 3

}	
//----------------------------------------------------------------------------------------------------------------------
// end Drive_PWMs
//----------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------
// Motor_Algorithm
// This routine executes a simple state machine which is designed to run at a fixed periodic rate
// It will calculate if the motor needs to speed up or slow down and stop at the desired position
// It assumes a motor that has 2 degrees per electrical revolution.  Other motors will need to be
// scaled based on a 2 deg / electrical revolution assumption.
// 
// IN:   motor paramaters
// OUT:  updated motor paramaters
//----------------------------------------------------------------------------------------------------------------------
void Motor_Algorithm(motorparams *mtr)
{
    u32  u32_PositionDifference;
    u32  u32_Position2zero;
    u16  u16_IntegerIterations;
    u16  u16_Decay;
    u16  u16_VEL_INC;
    u32  u32temp;
    u8   u8temp;
    u16  MAX_VEL_SCALED;


// The position angle is formatted to | 31 -- integer -- 16 | 15 -- fractional -- 0 |
#define SAMPLETIME_ms           1
#define MAX_ACCEL_deg_per_sec2  6000
#define SCALER                  (65536 * 1024 * SAMPLETIME_ms) / (1000 * (1000 / SAMPLE_RATE) )
#define VEL_INC                 MAX_ACCEL_deg_per_sec2 * 65536 * SAMPLETIME_ms / 1000 * SAMPLETIME_ms / 1000  *  SAMPLE_RATE * SAMPLE_RATE / 1000000
#define DECAY_STOP              4  
#define TABLE_SHIFT_FACTOR      SHIFT   // Based on the table size, the output of the state machine needs
                                        // to be shifted in order to access the correct table value
                                        // 12 - 32  point table 
                                        // 11 - 64  point table
                                        // 10 - 128 point table
                                        //  9 - 256 point table
                                        //  8 - 512 point table

    if (mtr->u16_cmdvel > mtr->u16_maxvel)
        mtr->u16_cmdvel = mtr->u16_maxvel;
    u32temp = ( SCALER * mtr->u16_cmdvel ) >> 10;
    MAX_VEL_SCALED =   (u16)(u32temp);


// Calculate the position difference
// The position of the motor is defined from 0 - 360 degrees.
// It is set up as a 32-bit variable that is formatted as below:
// ----------------------------------------------------
// |   Integer  (16-bits)    |   Fractional  16-bits  |
// ----------------------------------------------------

    u16_VEL_INC = VEL_INC;

    // Check commanded position and calculate commanded direction and position difference
    if ( mtr->u32_cmdpos >= mtr->u32_pos )
    {
        mtr->u8_cmddir = CW;
        u32_PositionDifference = mtr->u32_cmdpos - mtr->u32_pos;
    }
    else
    {
        mtr->u8_cmddir = CCW;
        u32_PositionDifference = mtr->u32_pos - mtr->u32_cmdpos;
    }


// If motor velocity = 0,  then set motor direction equal to commanded direction
    if (mtr->u16_vel == 0)
    {
        mtr->u8_dir = mtr->u8_cmddir;
    }

// Calculate the Integer Iterations

    u16_IntegerIterations =  (u16)( (u16)(mtr->u16_vel / u16_VEL_INC) + 1 );

        
// Calculate the Decay value
    if (mtr->u16_vel <= DECAY_STOP)
        u16_Decay = 0;
    else
        u16_Decay = mtr->u8_decayval; //        u16_Decay = DECAY_VAL;
    

// Calculate the position to zero speed.
    u32_Position2zero = (u32)(u16_IntegerIterations/2 + 1 + u16_Decay) * (u32)(mtr->u16_vel);

// Check whether to accelerate, decelerate or maintain speed
    if ( mtr->u8_dir != mtr->u8_cmddir)
    {
         if ( mtr->u16_vel >= u16_VEL_INC )       // if commanded direction <> motor direction, we need to decelerate
            mtr->u16_vel -= (u16)(u16_VEL_INC);
         else;
            mtr->u16_vel = 0;
    }
    else if ( u32_PositionDifference >= (u32_Position2zero + (u32)(VEL_INC*2) ) ) 
    {
        mtr->u16_vel += u16_VEL_INC;
        if ( mtr->u16_vel > MAX_VEL_SCALED)
            mtr->u16_vel = MAX_VEL_SCALED;
    }
    else if ( u16_IntegerIterations > 1 && u32_PositionDifference < u32_Position2zero )
    {
         if ( mtr->u16_vel >= u16_VEL_INC )      // if commanded direction <> motor direction, we need to decelerate
            mtr->u16_vel -= u16_VEL_INC;
         else
            mtr->u16_vel = 0;
    }  

// Move to next position
    if (mtr->u16_vel != 0)
    {
        if (mtr->u8_dir == CW)
            mtr->u32_pos += (u32)(mtr->u16_vel);
        else if (mtr->u16_vel <= mtr->u32_pos)
            mtr->u32_pos -= (u32)(mtr->u16_vel);        
        else
            mtr->u32_pos = 0;

    }
    else if (mtr->u8_dir == mtr->u8_cmddir)
        mtr->u32_pos = mtr->u32_cmdpos;
        
      
/*    if (mtr->u8_stopdirection == CCW)
        mtr->u8_step = ( mtr->u8_zerostep + (u8)( mtr->u32_pos >> TABLE_SHIFT_FACTOR ) ) & TABLEMASK; 
    else
        mtr->u8_step = ( mtr->u8_zerostep + ( TABLELENGTH - (u8)( mtr->u32_pos >> TABLE_SHIFT_FACTOR ) ) ) & TABLEMASK;
*/    

    u8temp = (u8)(mtr->u32_pos / TABLESCALEFACTOR);
    if (mtr->u8_stopdirection == CCW)
    {
        mtr->u8_step = ( mtr->u8_zerostep + u8temp );
        mtr->u8_step &= TABLEMASK;
    }
    else
    {
        mtr->u8_step = ( mtr->u8_zerostep - u8temp );
        mtr->u8_step &= TABLEMASK;
    }
}
//----------------------------------------------------------------------------------------------------------------------
// end Motor_Algorithm
//----------------------------------------------------------------------------------------------------------------------



//----------------------------------------------------------------------------------------------------------------------
// Move_Motor();
//      This function moves the motor based on the commanded position.
//      It calls Motor_Algorithm() and DrivePWMS()
//----------------------------------------------------------------------------------------------------------------------
void Move_Motor(motorparams *mtr)
{
  	  Motor_Algorithm(mtr);       // calculate next ustep  	
      Drive_PWMs(mtr);            // drive PWMs
}
//----------------------------------------------------------------------------------------------------------------------
// end Move_Motor();
//----------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------
// AlignMotorOnFullStep()
//
// This function will align the motor on a fullstep boundary.  
// It does so by moving to the closest fullstep towards the stop.
//----------------------------------------------------------------------------------------------------------------------
void AlignMotorOnFullStep(motorparams *mtr)
{
  
        if (mtr->u8_stopdirection == CCW)     // Check which direction the stop is located.
        {                                     // move motor CW (away from stop) to align
            if (mtr->u8_step > FULLSTEP3)     
                mtr->u8_step = FULLSTEP0;
            else if (mtr->u8_step > FULLSTEP0)
                mtr->u8_step = FULLSTEP1;
            else if (mtr->u8_step > FULLSTEP1)
                mtr->u8_step = FULLSTEP2;
            else if (mtr->u8_step > FULLSTEP2)
                mtr->u8_step = FULLSTEP3;
        }
        else                                  // move motor CCW (away from stop) to align
        {
            if (mtr->u8_step < FULLSTEP1)
                mtr->u8_step = FULLSTEP0;
            else if (mtr->u8_step < FULLSTEP2)
                mtr->u8_step = FULLSTEP1;
            else if (mtr->u8_step < FULLSTEP3)
                mtr->u8_step = FULLSTEP2;
            else if (mtr->u8_step < TABLELENGTH)
                mtr->u8_step = FULLSTEP3;
        }

        
        Drive_PWMs(mtr);   // drive to new step (drive motor PWMs)
}
//----------------------------------------------------------------------------------------------------------------------
// end AlignMotorOnFullStep
//----------------------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------------------
// RampUpMotor()
// This function ramps up motor speed for Microstep RTZ functionality
// Halfstep and Fullstep RTZ doesn't require motor ramp up.
//----------------------------------------------------------------------------------------------------------------------
void RampUpMotor(motorparams *mtr)
{  
    u16 i;
    
    if (mtr->u16RTZsteptime > mtr->u16RTZspeed)           // check if we are up to speed
    {
        mtr->DEBUG.speedmarker = 0;
        i = 0;
        
        // if startspeed is greater than first table entry we need to find next speed up on the ramp.
        while ( (MicroStepRTZramp[i] >= mtr->u16RTZsteptime ) && (MicroStepRTZramp[i++] != 0) );    
           
        if (MicroStepRTZramp[i] == 0)
            mtr->u16RTZsteptime = MicroStepRTZramp[i-1];  // if we try to go faster than ramp, limit to last item in ramp
        else
            mtr->u16RTZsteptime = MicroStepRTZramp[i];    // get next speed value out of ramp.                
                                                          
                                                          
        if (mtr->u16RTZsteptime < mtr->u16RTZspeed)       // also check if the speed from the accel table is faster than desired
            mtr->u16RTZsteptime = mtr->u16RTZspeed;       // if speed table value is faster than desired, set speed to desired
    }
    else  
    {
        mtr->DEBUG.speedmarker = 1;
        mtr->u16RTZsteptime = mtr->u16RTZspeed;           // if speed table value is faster than desired, set speed to desired
    }

}
//----------------------------------------------------------------------------------------------------------------------
// end RampUpMotor
//----------------------------------------------------------------------------------------------------------------------



//----------------------------------------------------------------------------------------------------------------------
// ClearDebugVars()
//
//----------------------------------------------------------------------------------------------------------------------
void ClearDebugVars(motorparams *mtr)
{

    mtr->s16RTZsampleZ0 = 0;
    mtr->s16RTZsampleZ1 = 0;
    mtr->s16RTZsampleZ2 = 0;
    mtr->s16RTZsampleZ3 = 0;

    //+++ DEBUG +++
    mtr->DEBUG.Coil2_diff = 0;
    mtr->DEBUG.Coil1_diff = 0;
    mtr->DEBUG.TotalValue = 0;
    mtr->DEBUG.TotalValue_diff = 0;
    //+++ DEBUG +++

}
//----------------------------------------------------------------------------------------------------------------------




//----------------------------------------------------------------------------------------------------------------------
// CheckStall 
// This routine takes the sample and checks if it is a successful stall condition.
//----------------------------------------------------------------------------------------------------------------------
void CheckStall(motorparams *mtr)
{
        // check the sample

        // DEBUG         
        if (mtr->u8_step == mtr->u8RTZsample_pt0)  {            
            mtr->DEBUG.FS0_diff = mtr->s16RTZvalue - mtr->DEBUG.FS0;
            mtr->DEBUG.FS0 = mtr->s16RTZvalue;
            mtr->DEBUG.Coil2_diff = mtr->s16RTZvalue - mtr->DEBUG.Coil2;
            mtr->DEBUG.Coil2 = mtr->s16RTZvalue;
            mtr->DEBUG.Fullstep = 0;                            
        } else if (mtr->u8_step == mtr->u8RTZsample_pt1)  {
            mtr->DEBUG.FS1_diff = mtr->s16RTZvalue - mtr->DEBUG.FS1;
            mtr->DEBUG.FS1 = mtr->s16RTZvalue;
            mtr->DEBUG.Coil1_diff = mtr->s16RTZvalue - mtr->DEBUG.Coil1;
            mtr->DEBUG.Coil1 = mtr->s16RTZvalue;          
            mtr->DEBUG.Fullstep = 1;          
        } else if (mtr->u8_step == mtr->u8RTZsample_pt2)  {
            mtr->DEBUG.FS2_diff = mtr->s16RTZvalue - mtr->DEBUG.FS2;
            mtr->DEBUG.FS2 = mtr->s16RTZvalue;
            mtr->DEBUG.Coil2_diff = mtr->s16RTZvalue - mtr->DEBUG.Coil2;
            mtr->DEBUG.Coil2 = mtr->s16RTZvalue;
            mtr->DEBUG.Fullstep = 2;          
        } else if (mtr->u8_step == mtr->u8RTZsample_pt3)  {
            mtr->DEBUG.FS3_diff = mtr->s16RTZvalue - mtr->DEBUG.FS3;
            mtr->DEBUG.FS3 = mtr->s16RTZvalue;
            mtr->DEBUG.Coil1_diff = mtr->s16RTZvalue - mtr->DEBUG.Coil1;
            mtr->DEBUG.Coil1 = mtr->s16RTZvalue;          
            mtr->DEBUG.Fullstep = 3;          
        }

        // DEBUG
        
        
        mtr->s16RTZsampleZ3 = mtr->s16RTZsampleZ2;
        mtr->s16RTZsampleZ2 = mtr->s16RTZsampleZ1;
        mtr->s16RTZsampleZ1 = mtr->s16RTZsampleZ0;
        mtr->s16RTZsampleZ0 = mtr->s16RTZvalue;
               
        mtr->s16RTZfilteredZ2 = mtr->s16RTZfilteredZ1;
        mtr->s16RTZfilteredZ1 = mtr->s16RTZfiltered;        
        mtr->s16RTZfiltered = (mtr->s16RTZsampleZ2 + 2*mtr->s16RTZsampleZ1 + mtr->s16RTZsampleZ0)>>2;
                        
        
        mtr->DEBUG.TotalValueZ2 = mtr->DEBUG.TotalValueZ1;
        mtr->DEBUG.TotalValueZ1 = mtr->DEBUG.TotalValue;
        mtr->DEBUG.TotalValue = (mtr->DEBUG.Coil1 + mtr->DEBUG.Coil2)>>1;
        mtr->DEBUG.TotalValue_diff = mtr->DEBUG.TotalValue - mtr->DEBUG.TotalValueZ1;
        mtr->DEBUG.TotalValue_diff2 = mtr->DEBUG.TotalValue - mtr->DEBUG.TotalValueZ2;
        
        
        if (mtr->u16RTZcount >=2) {
          mtr->s16RTZdiff = mtr->s16RTZfiltered - mtr->s16RTZfilteredZ1;
        }
        
        if (mtr->u16RTZcount >=3) {
          mtr->s16RTZdiff2 = mtr->s16RTZfiltered - mtr->s16RTZfilteredZ2;
        }

            

                    
        // make determination of stall


        if ( (mtr->s16RTZvalue == 0xffff) || (mtr->DEBUG.TotalValue_diff2 < mtr->s16RTZthreshold) )
            mtr->u8RTZdetected = 1;
        else
            mtr->u8RTZdetected = 0;
        

        if ( (mtr->s16RTZvalue == 0xffff) || (mtr->DEBUG.TotalValue_diff2 > (0-mtr->s16RTZthreshold) ) )        
            mtr->u8RTZflip = 1;
        else
            mtr->u8RTZflip = 0;
        
/*        if (mtr->DEBUG.TotalValue_diff < mtr->s16RTZthreshold)
            mtr->u8RTZflip = 1;
        else
            mtr->u8RTZflip = 0;
        
        if (mtr->DEBUG.TotalValue_diff > (0 - mtr->s16RTZthreshold) ) {
//          PTS_PTS0 = 1;
          mtr->u8RTZdetected = 1;
        }
        else {
//          PTS_PTS0 = 0;
          mtr->u8RTZdetected = 0;            
        }
*/
        
        // hold off on detection until desired time 
        // for high speed, this allows the motor to speed up.
        // for close proximity, this allows the filter to work.     
        
        if ( mtr->u16RTZcount >= mtr->u8RTZdetectdelay)
        {
            mtr->DEBUG.detectmarker = 1;  // marker to show detection decision                             
    
            if ( mtr->u8RTZflip || mtr->u8RTZdetected )
            {
        	        mtr->u8RTZon = RTZdone;             // if stall is u8RTZdetected, shut off the RTZ
                  mtr->u8_zerostep = mtr->u8_step;      // set the zero step to the ustep that we are on.
                  mtr->u32_pos = 0;                   // set the motor position to 0
                  mtr->u32_cmdpos = 0;                // set the commanded position to 0
                  Drive_PWMs(mtr);                          
                  mtr->DEBUG.detectmarker = 0;
            }  // end check for stall condition
        
        }
        else
            mtr->DEBUG.detectmarker = 0;                                                               
        

}
//----------------------------------------------------------------------------------------------------------------------
// end CheckStall
//----------------------------------------------------------------------------------------------------------------------



//==============================================================================================
//  Flyback_detect_Mx()
//      These functions implement a Flyback measurement for motors 0-3.  It does this by briefly
//      opening up either COIL1 or COIL2 based on the step_state.  It will then measure
//      the flyback decay. It will always measure the flyback decay on the negative side of the 
//      selected coil (i.e COIL1- or COIL2-).  However, in order to measure the flyback decay, the 
//      internal pull-device must be enabled to VDD or GND depending upon flyback polarity.  
//      For example, if there is a positive COIL current (current flowing into COIL-), the flyback
//      will be positive and the internal pull-down should be enabled.  This ensures that the input 
//      will transition through a logic level.
//
//      The step_state definitions are:
//      
//      0 = COIL1_IN  -->  Open COIL1 - Sample COIL1 - Enable Pull_down
//      1 = COIL1_OUT -->  Open COIL1 - Sample COIL1 - Enable Pull_up
//      2 = COIL2_IN  -->  Open COIL2 - Sample COIL2 - Enable Pull_down
//      3 = COIL2_OUT -->  Open COIL2 - Sample COIL2 - Enable Pull_up
//
//      To enable flyback measurement, the negative end of the COIL is connected to an input 
//      capture pin.  The positive end of the coil will default to GPIO when the MCPWM pins are
//      released and should be made an input if the pull device is enabled on the COIL minus pin.
//      If the pull device is not used the COIL positive connection can be made to an output and 
//      driven to either VDD or GND.
//
//      Connection Details:
//
//          GAUGE1 - M0_COS_M/PU0 -- IOC0_0
//          GAUGE1 - M0_COS_P/PU1 -- HI-Z   or  (Drive with PU1)
//          GAUGE1 - M0_SIN_M/PU2 -- IOC0_1
//          GAUGE1 - M0_SIN_P/PU3 -- HI-Z   or  (Drive with PU3)
//
//          GAUGE2 - M1_COS_M/PU4 -- IOC0_2
//          GAUGE2 - M1_COS_P/PU5 -- HI-Z   or  (Drive with PU5)
//          GAUGE2 - M1_SIN_M/PU6 -- IOC0_3
//          GAUGE2 - M1_SIN_P/PU7 -- HI-Z   or  (Drive with PU7)
//
//          GAUGE3 - M2_COS_M/PV0 -- IOC1_0
//          GAUGE3 - M2_COS_P/PV1 -- HI-Z   or  (Drive with PV1)
//          GAUGE3 - M2_SIN_M/PV2 -- IOC1_1
//          GAUGE3 - M2_SIN_P/PV3 -- HI-Z   or  (Drive with PV3)
//
//          GAUGE4 - M3_COS_M/PV4 -- IOC1_2
//          GAUGE4 - M3_COS_P/PV5 -- HI-Z   or  (Drive with PV5)
//          GAUGE4 - M3_SIN_M/PV6 -- IOC1_3
//          GAUGE4 - M3_SIN_P/PV7 -- HI-Z   or  (Drive with PV7)
//
//
//  FUNCTION DETAILS:
//
//      INPUT - uchar - step_state - this determins which COIL to sample 
//
//      OUTPUT - uint - flyback - this is the flyback measurement
//
//  
//===========================================================================================
void Flyback_detect_M0(motorparams *mtr)
{
	u8 tempTIOS;
	u8 input;
	u16 timeout;
	u16 elapsedtime;
	u16 first,second,flyback;

  tempTIOS = TIM0_TIOS; // keep copy of original timer channel settings
	TIM0_TIOS  &= 0xFC;	  // make sure IOC1_0 and IOC1_1 are set to be INPUT CAPTURES                                            


// ## 1  -  FIGURE OUT WHICH COIL IS BEING SAMPLED - MAKE SURE GPIO FOR COIL CONNECTIONS ARE CONFIGURED AS INPUT

	if ( (mtr->u8RTZcoilsample == COIL1_IN) || (mtr->u8RTZcoilsample == COIL1_OUT) ){
      PERU_PERU0 = 1;     //Enable pull device on COSM (PU0) for when the coil is released from MCPWM module
      DDRU_DDRU0 = 0;     //make sure COSM (PU0) is input for when the coil is released from MCPWM module
			DDRU_DDRU1 = 0;     //make sure COSP (PU1) is input for when the coil is released from MCPWM module    
			input = 0;	
	}                                              
	else if ( (mtr->u8RTZcoilsample == COIL2_IN ) || (mtr->u8RTZcoilsample == COIL2_OUT) ){
      PERU_PERU2 = 1;     //Enable pull device on COSM (PU2) for when the coil is released from MCPWM module
      DDRU_DDRU2 = 0;     //make sure COSM (PU2) is input for when the coil is released from MCPWM module
			DDRU_DDRU3 = 0;     //make sure COSP (PU3) is input for when the coil is released from MCPWM module
			input = 1;		                         
	}                                                 


// ## 2  -  DISABLE THE SELECTED COIL, DETERMINE FLYBACK POLARITY and MEASURE FLYBACK

	if (input == 0){		

      MCCTL0_MCTOIF = 1;                  // clear PWM overflow 
	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary - to allow for consistent charging of coil
		  MCCC0 = 0x00; 	                    // disable COIL1
      MCCTL0_MCTOIF = 1;                  // clear PWM overflow 
	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary - to allow for consistent charging of coil

    	first = TIM0_TCNT;                  // take first timer reading

      if (PTU_PTU0 == 0) {  // --- CHECK POLARITY OF FLYBACK ---
        PPSU_PPSU0 = 0;     //make sure COSM (PU0) has internal pull selected as vdd.
        TIM0_TCTL4 = 0x01;  //TIM0_CH0 capture on rising edges
      } else{
        PPSU_PPSU0 = 1;     //make sure COSM (PU0) has internal pull selected as gnd.
        TIM0_TCTL4 = 0x02;  //TIM0_CH0 capture on falling edges
      }

    	TIM0_TFLG1 = 0x01;                  // clear IC flag
      timeout = MAXFLYBACK;
      elapsedtime = 0;
      while ( ((TIM0_TFLG1 & 0x01) == 0)  && (elapsedtime < timeout) ) {
        elapsedtime = TIM0_TCNT - first;        
      } // wait for the flyback to end or timeout to occur
      second = TIM0_TC0;
	}
	else{		

      MCCTL0_MCTOIF = 1;                  // clear PWM overflow 
	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary - to allow for consistent charging of coil
		  MCCC1 = 0x00;	                      // disable COIL2
    	MCCTL0_MCTOIF = 1;                  // clear PWM overflow
 	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary - to allow for consistent charging of coil

    	first = TIM0_TCNT;                  // take first timer reading        

      if (PTU_PTU2 == 0) {  // --- CHECK POLARITY OF FLYBACK ---
        PPSU_PPSU2 = 0;     //make sure COSM (PU2) has internal pull selected as vdd.
        TIM0_TCTL4 = 0x04;  //TIM1_CH1 capture on rising edges
      } else{
        PPSU_PPSU2 = 1;     //make sure COSM (PU2) has internal pull selected as gnd.
        TIM0_TCTL4 = 0x08;  //TIM1_CH1 capture on falling edges
      }

      TIM0_TFLG1 = 0x02;                  // clear IC flag
      timeout = MAXFLYBACK;
      elapsedtime = 0;
      while ( ((TIM0_TFLG1 & 0x02) == 0)  && (elapsedtime < timeout) ){
        elapsedtime = TIM0_TCNT - first;        
      } // wait for the flyback to end or timeout to occur
      second = TIM0_TC1;
	}   
    

//  ## 3  -  RE-ENABLE THE COILS    
  
  MCCC0 = 0xF0;     
  MCCC1 = 0xF0;
	TIM0_TIOS |= tempTIOS;	                // set timers back to original setting
   

//  ## 4  -  CALCULATE FLYBACK TIME

  flyback = second - first;               // calculate flyback time

  if (timeout != 0)
      mtr->s16RTZvalue = flyback;         // return the value
	else
	    mtr->s16RTZvalue = 65535;
	
}


//===========================================================================================


void Flyback_detect_M1(motorparams *mtr)
{
	u8 tempTIOS;
	u8 input;
	u16 timeout;
	u16 elapsedtime;
	u16 first,second,flyback;

  tempTIOS = TIM0_TIOS; // keep copy of original timer channel settings
	TIM0_TIOS  &= 0xF3;	  // make sure IOC0_2 and IOC0_3 are set to be INPUT CAPTURES                                            


// ## 1  -  FIGURE OUT WHICH COIL IS BEING SAMPLED - MAKE SURE GPIO FOR COIL CONNECTIONS ARE CONFIGURED AS INPUT

	if ( (mtr->u8RTZcoilsample == COIL1_IN) || (mtr->u8RTZcoilsample == COIL1_OUT) ){
      PERU_PERU4 = 1;     //Enable pull device on COSM (PU4) for when the coil is released from MCPWM module
      DDRU_DDRU4 = 0;     //make sure COSM (PU4) is input for when the coil is released from MCPWM module
			DDRU_DDRU5 = 0;     //make sure COSP (PU5) is input for when the coil is released from MCPWM module    
			input = 2;	
	}                                              
	else if ( (mtr->u8RTZcoilsample == COIL2_IN ) || (mtr->u8RTZcoilsample == COIL2_OUT) ){
      PERU_PERU6 = 1;     //Enable pull device on COSM (PU6) for when the coil is released from MCPWM module
      DDRU_DDRU6 = 0;     //make sure COSM (PU6) is input for when the coil is released from MCPWM module
			DDRU_DDRU7 = 0;     //make sure COSP (PU7) is input for when the coil is released from MCPWM module
			input = 3;		                         
	}                                                 


// ## 2  -  DISABLE THE SELECTED COIL, DETERMINE FLYBACK POLARITY and MEASURE FLYBACK

	if (input == 2){		

    	MCCTL0_MCTOIF = 1;                  // clear PWM overflow
 	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary 
		  MCCC2 = 0x00; 	                    // disable the cos coil
     	MCCTL0_MCTOIF = 1;                  // clear PWM overflow 
	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary 
      
    	first = TIM0_TCNT;                  // take first reading
      if (PTU_PTU4 == 0) {  // --- CHECK POLARITY OF FLYBACK ---
        PPSU_PPSU4 = 0;     //make sure COSM (PU4) has internal pull selected as vdd.
        TIM0_TCTL4 = 0x10;  //TIM0_CH2 capture on rising edges
      } else{
        PPSU_PPSU4 = 1;     //make sure COSM (PU4) has internal pull selected as gnd.
        TIM0_TCTL4 = 0x20;  //TIM0_CH2 capture on falling edges
      }

    	TIM0_TFLG1 = 0x04;                  // clear IC flag
      timeout = MAXFLYBACK;
      elapsedtime = 0;
      while ( ((TIM0_TFLG1 & 0x04) == 0)  && (elapsedtime < timeout) ) {
        elapsedtime = TIM0_TCNT - first;        
      } // wait for the flyback to end or timeout to occur
      second = TIM0_TC2;
	}
	else{		

    	MCCTL0_MCTOIF = 1;                  // clear PWM overflow
 	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary 
		  MCCC3 = 0x00;	                      // disable COIL2
    	MCCTL0_MCTOIF = 1;                  // clear PWM overflow
 	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary - to allow for consistent charging of coil

    	first = TIM0_TCNT;                  // take first timer reading        

      if (PTU_PTU6 == 0) {  // --- CHECK POLARITY OF FLYBACK ---
        PPSU_PPSU6 = 0;     //make sure COSM (PU6) has internal pull selected as vdd.
        TIM0_TCTL4 = 0x40;  //TIM0_CH3 capture on rising edges
      } else{
        PPSU_PPSU6 = 1;     //make sure COSM (PU6) has internal pull selected as gnd.
        TIM0_TCTL4 = 0x80;  //TIM0_CH3 capture on falling edges
      }

      TIM0_TFLG1 = 0x08;                  // clear IC flag
      timeout = MAXFLYBACK;
      elapsedtime = 0;
      while ( ((TIM0_TFLG1 & 0x08) == 0)  && (elapsedtime < timeout) ){
        elapsedtime = TIM0_TCNT - first;        
      } // wait for the flyback to end or timeout to occur
      second = TIM0_TC3;
	}   
    

//  ## 3  -  RE-ENABLE THE COILS    
  
  MCCC2 = 0xF0;     
  MCCC3 = 0xF0;
	TIM0_TIOS |= tempTIOS;	                // set timers back to original setting
   

//  ## 4  -  CALCULATE FLYBACK TIME

  flyback = second - first;               // calculate flyback time

  if (timeout != 0)
      mtr->s16RTZvalue = flyback;         // return the value
	else
	    mtr->s16RTZvalue = 65535;
	
}


//===========================================================================================
    

void Flyback_detect_M2(motorparams *mtr)
{
	u8 tempTIOS;
	u8 input;
	u16 timeout;
	u16 elapsedtime;
	u16 first,second,flyback;

  tempTIOS = TIM1_TIOS; // keep copy of original timer channel settings
	TIM1_TIOS  &= 0xFC;	  // make sure IOC1_0 and IOC1_1 are set to be INPUT CAPTURES                                            


// ## 1  -  FIGURE OUT WHICH COIL IS BEING SAMPLED - MAKE SURE GPIO FOR COIL CONNECTIONS ARE CONFIGURED AS INPUT

	if ( (mtr->u8RTZcoilsample == COIL1_IN) || (mtr->u8RTZcoilsample == COIL1_OUT) ){
      PERV_PERV0 = 1;     //Enable pull device on COSM (PV0) for when the coil is released from MCPWM module
      DDRV_DDRV0 = 0;     //make sure COSM (PV0) is input for when the coil is released from MCPWM module
			DDRV_DDRV1 = 0;     //make sure COSP (PV1) is input for when the coil is released from MCPWM module    
			input = 0;	
	}                                              
	else if ( (mtr->u8RTZcoilsample == COIL2_IN ) || (mtr->u8RTZcoilsample == COIL2_OUT) ){
      PERV_PERV2 = 1;     //Enable pull device on COSM (PV2) for when the coil is released from MCPWM module
      DDRV_DDRV2 = 0;     //make sure COSM (PV2) is input for when the coil is released from MCPWM module
			DDRV_DDRV3 = 0;     //make sure COSP (PV3) is input for when the coil is released from MCPWM module
			input = 1;		                         
	}                                                 


// ## 2  -  DISABLE THE SELECTED COIL, DETERMINE FLYBACK POLARITY and MEASURE FLYBACK

	if (input == 0){		

      MCCTL0_MCTOIF = 1;                  // clear PWM overflow 
	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary - to allow for consistent charging of coil
		  MCCC4 = 0x00; 	                    // disable COIL1
      MCCTL0_MCTOIF = 1;                  // clear PWM overflow 
	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary - to allow for consistent charging of coil

    	first = TIM1_TCNT;                  // take first timer reading

      if (PTV_PTV0 == 0) {  // --- CHECK POLARITY OF FLYBACK ---
        PPSV_PPSV0 = 0;     //make sure COSM (PV0) has internal pull selected as vdd.
        TIM1_TCTL4 = 0x01;  //TIM1_CH0 capture on rising edges
      } else{
        PPSV_PPSV0 = 1;     //make sure COSM (PV0) has internal pull selected as gnd.
        TIM1_TCTL4 = 0x02;  //TIM1_CH0 capture on falling edges
      }

    	TIM1_TFLG1 = 0x01;                  // clear IC flag
      timeout = MAXFLYBACK;
      elapsedtime = 0;
      while ( ((TIM1_TFLG1 & 0x01) == 0)  && (elapsedtime < timeout) ) {
        elapsedtime = TIM1_TCNT - first;        
      } // wait for the flyback to end or timeout to occur
      second = TIM1_TC0;
	}
	else{		

      MCCTL0_MCTOIF = 1;                  // clear PWM overflow 
	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary - to allow for consistent charging of coil
		  MCCC5 = 0x00;	                      // disable COIL2
    	MCCTL0_MCTOIF = 1;                  // clear PWM overflow
 	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary - to allow for consistent charging of coil

    	first = TIM1_TCNT;                  // take first timer reading        

      if (PTV_PTV2 == 0) {  // --- CHECK POLARITY OF FLYBACK ---
        PPSV_PPSV2 = 0;     //make sure COSM (PV2) has internal pull selected as vdd.
        TIM1_TCTL4 = 0x04;  //TIM1_CH1 capture on rising edges
      } else{
        PPSV_PPSV2 = 1;     //make sure COSM (PV2) has internal pull selected as gnd.
        TIM1_TCTL4 = 0x08;  //TIM1_CH1 capture on falling edges
      }

      TIM1_TFLG1 = 0x02;                  // clear IC flag
      timeout = MAXFLYBACK;
      elapsedtime = 0;
      while ( ((TIM1_TFLG1 & 0x02) == 0)  && (elapsedtime < timeout) ){
        elapsedtime = TIM1_TCNT - first;        
      } // wait for the flyback to end or timeout to occur
      second = TIM1_TC1;
	}   
    

//  ## 3  -  RE-ENABLE THE COILS    
  
  MCCC4 = 0xF0;     
  MCCC5 = 0xF0;
	TIM1_TIOS |= tempTIOS;	                // set timers back to original setting
   

//  ## 4  -  CALCULATE FLYBACK TIME

  flyback = second - first;               // calculate flyback time

  if (timeout != 0)
      mtr->s16RTZvalue = flyback;         // return the value
	else
	    mtr->s16RTZvalue = 65535;
	
}


//===========================================================================================


void Flyback_detect_M3(motorparams *mtr)
{
	u8 tempTIOS;
	u8 input;
	u16 timeout;
	u16 elapsedtime;
	u16 first,second,flyback;

  tempTIOS = TIM1_TIOS; // keep copy of original timer channel settings
	TIM1_TIOS  &= 0xF3;	  // make sure IOC1_2 and IOC1_3 are set to be INPUT CAPTURES                                            


// ## 1  -  FIGURE OUT WHICH COIL IS BEING SAMPLED - MAKE SURE GPIO FOR COIL CONNECTIONS ARE CONFIGURED AS INPUT

	if ( (mtr->u8RTZcoilsample == COIL1_IN) || (mtr->u8RTZcoilsample == COIL1_OUT) ){
      PERV_PERV4 = 1;     //Enable pull device on COSM (PV4) for when the coil is released from MCPWM module
      DDRV_DDRV4 = 0;     //make sure COSM (PV4) is input for when the coil is released from MCPWM module
			DDRV_DDRV5 = 0;     //make sure COSP (PV5) is input for when the coil is released from MCPWM module    
			input = 2;	
	}                                              
	else if ( (mtr->u8RTZcoilsample == COIL2_IN ) || (mtr->u8RTZcoilsample == COIL2_OUT) ){
      PERV_PERV6 = 1;     //Enable pull device on COSM (PV6) for when the coil is released from MCPWM module
      DDRV_DDRV6 = 0;     //make sure COSM (PV6) is input for when the coil is released from MCPWM module
			DDRV_DDRV7 = 0;     //make sure COSP (PV7) is input for when the coil is released from MCPWM module
			input = 3;		                         
	}                                                 


// ## 2  -  DISABLE THE SELECTED COIL, DETERMINE FLYBACK POLARITY and MEASURE FLYBACK

	if (input == 2){		

      MCCTL0_MCTOIF = 1;                  // clear PWM overflow 
	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary - to allow for consistent charging of coil
		  MCCC6 = 0x00; 	                    // disable COIL1
      MCCTL0_MCTOIF = 1;                  // clear PWM overflow 
	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary - to allow for consistent charging of coil

    	first = TIM1_TCNT;                  // take first timer reading

      if (PTV_PTV4 == 0) {  // --- CHECK POLARITY OF FLYBACK ---
        PPSV_PPSV4 = 0;     //make sure COSM (PV4) has internal pull selected as vdd.
        TIM1_TCTL4 = 0x10;  //TIM1_CH2 capture on rising edges
      } else{
        PPSV_PPSV4 = 1;     //make sure COSM (PV4) has internal pull selected as gnd.
        TIM1_TCTL4 = 0x20;  //TIM1_CH2 capture on falling edges
      }

    	TIM1_TFLG1 = 0x04;                  // clear IC flag
      timeout = MAXFLYBACK;
      elapsedtime = 0;
      while ( ((TIM1_TFLG1 & 0x04) == 0)  && (elapsedtime < timeout) ) {
        elapsedtime = TIM1_TCNT - first;        
      } // wait for the flyback to end or timeout to occur
      second = TIM1_TC2;
	}
	else{		

      MCCTL0_MCTOIF = 1;                  // clear PWM overflow 
	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary - to allow for consistent charging of coil
		  MCCC7 = 0x00;	                      // disable COIL2
    	MCCTL0_MCTOIF = 1;                  // clear PWM overflow
 	    while (MCCTL0_MCTOIF == 0);         // wait for PWM to overflow - PWM boundary - to allow for consistent charging of coil

    	first = TIM1_TCNT;                  // take first timer reading        

      if (PTV_PTV6 == 0) {  // --- CHECK POLARITY OF FLYBACK ---
        PPSV_PPSV6 = 0;     //make sure COSM (PV6) has internal pull selected as vdd.
        TIM1_TCTL4 = 0x40;  //TIM1_CH3 capture on rising edges
      } else{
        PPSV_PPSV6 = 1;     //make sure COSM (PV6) has internal pull selected as gnd.
        TIM1_TCTL4 = 0x80;  //TIM1_CH3 capture on falling edges
      }

      TIM1_TFLG1 = 0x08;                  // clear IC flag
      timeout = MAXFLYBACK;
      elapsedtime = 0;
      while ( ((TIM1_TFLG1 & 0x08) == 0)  && (elapsedtime < timeout) ){
        elapsedtime = TIM1_TCNT - first;        
      } // wait for the flyback to end or timeout to occur
      second = TIM1_TC3;
	}   
    

//  ## 3  -  RE-ENABLE THE COILS    
  
  MCCC6 = 0xF0;     
  MCCC7 = 0xF0;
	TIM1_TIOS |= tempTIOS;	                // set timers back to original setting
   

//  ## 4  -  CALCULATE FLYBACK TIME

  flyback = second - first;               // calculate flyback time

  if (timeout != 0)
      mtr->s16RTZvalue = flyback;         // return the value
	else
	    mtr->s16RTZvalue = 65535;
	
}


//===========================================================================================



//----------------------------------------------------------------------------------------------------------------------
// MicrostepRTZ():
//      This function is used to determine when and which coil to sample for Microstep RTZ detection.  
//      It assumes a 16 pt microstep waveform.  The coils are sampled on a full-step boundary.  
//      Therefore, there are 3 microsteps of movement before a sample is taken.  
//      For best detection, the motor should be stopped on a full-step boundary before the RTZ function 
//      is initiated.
//
//        |<---   16  microsteps    --->|
//      O x x x O x x x O x x x O x x x O
//              ^       ^       ^       ^
//              |       |       |       |
//                        sample
//
//
//      The coil sampling decision is dependent upon the direction of motor movement during detection.
//
//      Below is the coil voltages and sampling logic
//          COIL1_xxx - sample COIL1 
//          COIL1_IN - sample COIL2 
//          COILx_NEG - current flowing from COIL connection - results in positive flyback voltage
//          COILx_POS - current flowing into COIL connection - results in negative flyback voltage
//
//      NOTE: sampling logic is dependent upon which end of the COIL is sampled.  See graphic below.
//
// .....................................................................................................................  
//
//   <----- CCW stall detection
//
//               | FS0 |                 | FS1 |                 | FS2 |                 | FS3 |
//  samplestate  |  0  |  1  |  2  |  3  |  4  |  5  |  6  |  7  |  8  |  9  | 10  | 11  | 12  | 13  | 14  | 15  |  0  |
//                           | HS0 |                 | HS1 |                 | HS2 |                 | HS3 |   
//                _____                                                                                           _____
//               |     |_____                                                                               _____|     |
//                           |_____                                                                   _____|
//   COIL1                         |                                                                 |
//   (UR)                          |_____                                                       _____|
//   (COS)                               |                                                     |
//                                       |                                                     |
//                                       |_____                                           _____|
//                                          ^  |                                         |  ^
//                                          |  |                                         |  |
//                                          |  |_____                               _____|  |
//                                          |        |                             |        |
//                                          |        |_____                   _____|        |
//                                          |              |_____       _____|              |
//                                          |                    |_____|                    |
//                                      COIL1_IN                                       COIL1_OUT   -- if sampling positive side of COIL
//                                      COIL1_OUT                                       COIL1_IN   -- if sampling negative side of COIL
//
//              COIL2_OUT                                       COIL2_IN                           -- if sampling positive side of COIL
//              COIL2_IN                                       COIL2_OUT                           -- if sampling negative side of COIL
//                  |                     _____                     |
//                  |               _____|     |_____               |                                                        
//                  |         _____|                 |_____         |                                                        
//   COIL2          |        |                             |        |                                                        
//   (UL)           |   _____|                             |_____   |                                                    
//   (SIN)          |  |                                         |  |                                                   
//                  |  |                                         |  |                                                   
//                __v__|                                         |__v__                                           _____|
//               |                                                     |                                         |
//                                                                     |                                         |
//                                                                     |_____                               _____|
//                                                                           |                             |
//                                                                           |_____                   _____|
//                                                                                 |_____       _____|
//                                                                                       |_____|
//
// .....................................................................................................................  
//
//   CW stall detection ----->
//
//               | FS0 |                 | FS1 |                 | FS2 |                 | FS3 |                 | FS0 |
//  samplestate  |  0  |  1  |  2  |  3  |  4  |  5  |  6  |  7  |  8  |  9  | 10  | 11  | 12  | 13  | 14  | 15  |  0  |
//                           | HS0 |                 | HS1 |                 | HS2 |                 | HS3 |   
//                _____                                                                                           _____
//               |     |_____                                                                               _____|     |
//                           |_____                                                                   _____|
//   COIL1                         |                                                                 |
//   (UR)                          |_____                                                       _____|
//   (COS)                               |                                                     |
//                                       |                                                     |
//                                       |_____                                           _____|
//                                          ^  |                                         |  ^
//                                          |  |                                         |  |
//                                          |  |_____                               _____|  |
//                                          |        |                             |        |
//                                          |        |_____                   _____|        |
//                                          |              |_____       _____|              |
//                                          |                    |_____|                    |
//                                      COIL1_OUT                                       COIL1_IN   -- if sampling positive side of COIL
//                                      COIL1_IN                                       COIL1_OUT   -- if sampling negative side of COIL
//
//              COIL2_IN                                       COIL2_OUT                           -- if sampling positive side of COIL
//              COIL2_OUT                                       COIL2_IN                           -- if sampling negative side of COIL
//                  |                     _____                     |
//                  |               _____|     |_____               |                                                        
//                  |         _____|                 |_____         |                                                        
//   COIL2          |        |                             |        |                                                        
//   (UL)           |   _____|                             |_____   |                                                    
//   (SIN)          |  |                                         |  |                                                   
//                  |  |                                         |  |                                                   
//                __v__|                                         |__v__                                           _____|
//               |                                                     |                                         |
//                                                                     |                                         |
//                                                                     |_____                               _____|
//                                                                           |                             |
//                                                                           |_____                   _____|
//                                                                                 |_____       _____|
//                                                                                       |_____|
//
//----------------------------------------------------------------------------------------------------------------------
u16 MicrostepRTZ(motorparams *mtr)
{
    
    u8 sample;

    // Check to see if this is the beginning of the RTZ function
    // if this is the start of RTZ, set the steptime to the desired start speed.
  
    if (mtr->u16RTZcount == 0xFFFF)                
    {                
                 
        ClearDebugVars(mtr);          
        mtr->u8RTZflip = 0;
        mtr->u8RTZdetected = 0;

        mtr->u16RTZcount++;
    
        mtr->u16RTZsteptime = mtr->u16RTZstartspeed;    // set startspeed        
        return(mtr->u16RTZsteptime);   
    }
    
 
    // Motor has been aligned to a FullStep and can now be stepped for RTZ detection
    else
    {
          mtr->DEBUG.microstepcount--;                                                                                                             
         

          if (mtr->u8_stopdirection == CW)
          {    		
                  // --- CW RTZ ---
                        
                mtr->u8_step += mtr->u8RTZincrementer;   
                mtr->u8_step &= TABLEMASK;

      	  } 
    	
        	else 
        	{			
              
              // --- CCW RTZ ---
              			
                mtr->u8_step -= mtr->u8RTZincrementer;
                mtr->u8_step &= TABLEMASK;
                
      	  }



/*          mtr->u8RTZsample_pt0 = SAMPLE0;
          mtr->u8RTZsample_pt1 = SAMPLE1;
          mtr->u8RTZsample_pt2 = SAMPLE2;
          mtr->u8RTZsample_pt3 = SAMPLE3;
*/
                
          if (mtr->u8_step == mtr->u8RTZsample_pt0){
        			mtr->u8RTZcoilsample = COIL1_POS_CURRENT;
        		  sample = 1;
          }
          else if (mtr->u8_step == mtr->u8RTZsample_pt1){
        			mtr->u8RTZcoilsample = COIL2_POS_CURRENT;
        		  sample = 1;
          }
          else if (mtr->u8_step == mtr->u8RTZsample_pt2){
        			mtr->u8RTZcoilsample = COIL1_NEG_CURRENT;
        		  sample = 1;
          } 
          else if (mtr->u8_step == mtr->u8RTZsample_pt3){
        			mtr->u8RTZcoilsample = COIL2_NEG_CURRENT;
        		  sample = 1;
          } 
          else{
              sample = 0;
          }

                           
          Drive_PWMs(mtr);                          
          if (sample == 1)     
          {            
              mtr->u16RTZcount++;
              
              if (mtr->u8_pwmchan == 0)
                  Flyback_detect_M0(mtr);                     
              else if (mtr->u8_pwmchan == 1)                
                  Flyback_detect_M1(mtr);                     
              else if (mtr->u8_pwmchan == 2)                
                  Flyback_detect_M2(mtr);                     
              else
                  Flyback_detect_M3(mtr);                    
              
              CheckStall(mtr);
          }      
               
          RampUpMotor(mtr);  // ramp up motor speed if needed 
        
          return(mtr->u16RTZsteptime);
      }
  
}
//----------------------------------------------------------------------------------------------------------------------
// end MicroStepRTZ
//----------------------------------------------------------------------------------------------------------------------





//----------------------------------------------------------------------------------------------------------------------
// MotorScheduler()
// This function determines if we are doing normal motor movement or performing an RTZ function.
// During normal motor movement, this function is called every 1ms.  
// During microstep RTZ the time is variable based on the microstep time.
//----------------------------------------------------------------------------------------------------------------------
u16 MotorScheduler(motorparams *mtr)
{
    u16 nextsteptime;

    if (mtr->u8RTZon == RTZdone)
    { 
        // execute normal motor movement
        
        Move_Motor(mtr);  //
        nextsteptime = (SAMPLE_RATE * TIMERCLK);

    } 
    
    
    else
    {
        // execute RTZ 

        nextsteptime = MicrostepRTZ(mtr);      // Note: next step time is in milliseconds so we need to scale
        nextsteptime *= TIMERCLK;                              // Microstep Ramp is in us so we need to scale it by the timer resolution

    }
                                                               
    return(nextsteptime);    
}
//----------------------------------------------------------------------------------------------------------------------
// end MotorScheduler()
//----------------------------------------------------------------------------------------------------------------------




#pragma CODE_SEG NON_BANKED


//==================/\/\/\/\/ INTERRUPT \/\/\/\/\===================
// This interrupt service routine is used to schedule the motor 
// movement for M0.  
// It is called every 1.000ms for normal motor movement and will be
// variable during RTZ functionality.
//==================================================================
__interrupt VectorNumber_Vtim0ch0 void _isr_M0__IOC_0_Interrupt (void)
{

   	TIM0_TC0 = TIM0_TC0 + MotorScheduler(&Motor0);    // set next step time        
  	TIM0_TFLG1 = 0x01;                              // clear OC0 interrupt flag    

}	// end TCH0_OC_INT()

//==================/\/\/\/\/ INTERRUPT \/\/\/\/\===================


//==================/\/\/\/\/ INTERRUPT \/\/\/\/\===================
// This interrupt service routine is used to schedule the motor 
// movement for M1.  
// It is called every 1.000ms for normal motor movement and will be
// variable during RTZ functionality.
//==================================================================
__interrupt VectorNumber_Vtim0ch2 void _isr_M1__IOC_1_Interrupt (void)
{

   	TIM0_TC2 = TIM0_TC2 + MotorScheduler(&Motor1);    // set next step time
  	TIM0_TFLG1 = 0x04;	                            // clear interrupt flag 

} // end TCH1_OC_INT() 

//==================/\/\/\/\/ INTERRUPT \/\/\/\/\===================


//==================/\/\/\/\/ INTERRUPT \/\/\/\/\===================
// This interrupt service routine is used to schedule the motor 
// movement for M2.  
// It is called every 1.000ms for normal motor movement and will be
// variable during RTZ functionality.
//==================================================================
__interrupt VectorNumber_Vtim1ch0 void _isr_M2__IOC_0_Interrupt (void)
{   
    DDRS = 0xFF;
    PTS_PTS0 = 1;
    
   	TIM1_TC0 = TIM1_TC0 + MotorScheduler(&Motor2);    // set next step time        
  	TIM1_TFLG1 = 0x01;	                            // clear OC0 interrupt flag    

    PTS_PTS0 = 0;
}	// end TCH0_OC_INT()

//==================/\/\/\/\/ INTERRUPT \/\/\/\/\===================


//==================/\/\/\/\/ INTERRUPT \/\/\/\/\===================
// This interrupt service routine is used to schedule the motor 
// movement for M3.  
// It is called every 1.000ms for normal motor movement and will be
// variable during RTZ functionality.
//==================================================================
__interrupt VectorNumber_Vtim1ch2 void _isr_M3__IOC_1_Interrupt (void)
{
    
   	TIM1_TC2 = TIM1_TC2 + MotorScheduler(&Motor3);    // set next step time
  	TIM1_TFLG1 = 0x04;	                            // clear interrupt flag 

} // end TCH1_OC_INT() 

//==================/\/\/\/\/ INTERRUPT \/\/\/\/\===================



//==================/\/\/\/\/ INTERRUPT \/\/\/\/\===================
// This interrupt service routine is used to enable a 1ms sytem timer.
// It uses the Motor Control Timer overflow to set the timer.
// The overflow interupt occurs every 32us
//==================================================================
__interrupt VectorNumber_Vtim0ch4 void _isr_SystemTick_Interrupt (void)
{
  	TIM0_TFLG1 = 0x10;	                      // clear interrupt flag 
   	TIM0_TC4 = TIM0_TC4 + _1ms;

   	u16SysTimer++;   // create main system timer 

} // end TCH0_OC_INT() 

//==================/\/\/\/\/ INTERRUPT \/\/\/\/\===================


//==================/\/\/\/\/ INTERRUPT \/\/\/\/\===================
// This interrupt service routine controls the switch on the demo board
// it enabled both right and left indicators and rotates the Odo/tripA/B 
// it also incoporates a hazard function on pressing SW4 and SW5 simultaneously
//==================================================================     
__interrupt VectorNumber_Vportad void _isr_SW6__Interrupt (void)
  {
     
     extern count,cruise;
     int j ; 
     
extern UINT16  tripA_value, tripB_value;              // 1M miles requires at least 20 bits: 
extern UINT32  odometer_value;                        // 1M  = 1111 0100 0010 0100 0000  
     
  
       // SW6 MENU SWITCH
    if (PT1AD_PT1AD5 == 0)                            // used to toggle between ODO/TRIPA/TRIP B on PTS5 press
      {
        count=count++;
        if (count == 3)
          { 
            count=0; 
          }
        MDelay(1000);                                 // do nothing - debounce for SW6     
                                                      
      switch(count)                                   // display the correct title/number on the button press immediantly 
      {                                               // (therfore not waiting on program polling through main while loop to update)
        // ODOMETER
        case 0:
                
                SET_ODO_TITLE (ON,ON,OFF,OFF,OFF);   // (ODO, MILES, TRIPA, TRIPB, KM) -->SEGs
                
                SET_ODO_DIGITS(ON,odometer_value);   // Update odometer digits with global odometer value
                break;
                      
        //TRIP A Meter
        case 1: 
                SET_ODO_TITLE (OFF,ON,ON,OFF,OFF);   // (ODO, MILES, TRIPA, TRIPB, KM) -->SEGs
               
               SET_TRIPA_DIGITS(ON,tripA_value);    // Update tripA digits with global tripA value
           
                
                break;
        
        //TRIP B meter
        case 2: 
                
                SET_ODO_TITLE (OFF,ON,OFF,ON,OFF);    // (ODO, MILES, TRIPA, TRIPB, KM) -->SEGs
                SET_TRIPB_DIGITS(ON,tripB_value);     // Update tripB digits with global odometer value
                break;
      }
    
                PIF1AD_PIF1AD5	= 1;				        	     // clear PT1AD5  interrupt flag - Menu Switch  
  }
  
  //  SW4 RIGHT INDICATOR
  if (PT1AD_PT1AD6 == 0)                 
        
     {
        cruise=cruise++;
        if (cruise >= 2)
          { 
            cruise=0; 
          }
        MDelay(1000);                                 // do nothing - debounce for SW6     
                                                      
      switch(cruise)                                       // display the correct title/number on the button press immediantly 
      {                                               // (therfore not waiting on program polling through main while loop to update)
        case 0:                                       // Cruise control engaged
               SEG_S4 = 1;
                break;
                      
        case 1:                                       // cruise control off.
                SEG_S4=0;
                
                break;

      }
                PIF1AD_PIF1AD6	= 1;				        	     // clear PT1AD6  interrupt flag - Menu Switch  
  }

 
    // hazard - warning      
     while (PT1AD_PT1AD7 == 0 && PT1AD_PT1AD6 == 0 )                               
         { 
           unsigned char  w, x, y, z;	/*Simple loop count variables*/ 
            MDelay(2000);
            PTR ^=0x0F;
            PTS ^=0x70;
            HAZARD_TITLE();                           // display flashing 'hazard' on LCD
            
  
     		for(w=0; w<=1; w++) 				/*Beep 2 times*/
			{
				for(x=0xFE; x>=0x01; x--)		/*Fade control loop*/
				{
			  	PWMPER3 = x;		/*Load PWM period register (PWM period variation)*/

				for (y=0; y<=x; y++)		/*Variable loop to generate up slope of sinusoid*/
					{					
						PWMPER3 = y;	/*Loads PWM duty register*/
			//			MDelay0(10);				/*Optional delay. See above*/
			 		}
	  				  	
			  		for (y=0xFF; y>=x; y--)		/*Variable loop to generate down slope of sinusoid*/
			  		{
			  		PWMPER3 = y;	/*Loads PWM duty register*/
			  //			MDelay0(10);				/*Optional delay. See above*/
			  		}	
				}
	  //			MDelay0(1);					/*Time between beeps. O gives continuous tone, higher values give greater times*/
	  		}
          
       //re-initalise the pointer lighting
          
 }
 
   PTR_PTR3 = 1;                                   // re-enable LED shaft lighting 
       PTS = 0x70;                                    // re-enable LED shaft lighting
       PTR_PTR2 = 0;                                  // clear hi beam light
       PIF1AD	= 0xC0;				        	                // clear Indicator interrupt flag - Menu Switch 
#pragma CODE_SEG DEFAULT
}









