#ifndef MOTOR_ALGO_H
#define MOTOR_ALGO_H

#include "cpu.h"

#define OSCCLK 8   //Oscillator Clock in MHz
#define BUSCLK 8   //Bus Clock in MHz
#define TIMERCLK 8 //Timer Clock in MHz
#define TABLELENGTH 128
#define SAMPLE_RATE 1000

#define TABLEMASK   TABLELENGTH - 1         // this is used as a MASK for the table pointer 

#define HY
#define MEASURE NEGATIVE_TERMINAL

#if ( (BUSCLK / TIMERCLK) == 1)
  #define TIMERPRESCALER 0
#endif  

#if ( (BUSCLK / TIMERCLK) == 2)
  #define TIMERPRESCALER 1
#endif  

#if ( (BUSCLK / TIMERCLK) == 4)
  #define TIMERPRESCALER 2
#endif  

#if ( (BUSCLK / TIMERCLK) == 8)
  #define TIMERPRESCALER 3
#endif  

#if ( (BUSCLK / TIMERCLK) == 16)
  #define TIMERPRESCALER 4
#endif  

#define _50us     50 * TIMERCLK
#define _250us   250 * TIMERCLK
#define _500us   500 * TIMERCLK
#define _1ms    1000 * TIMERCLK
#define _2ms    2000 * TIMERCLK
#define _5ms    5000 * TIMERCLK
#define _10ms  10000 * TIMERCLK

#define DEFAULT_USTEPS_IN_CYCLE  64                               // number of microsteps in an electrical cycle

#define HALFSTEP  TABLELENGTH / 8   // 8 Halfsteps in an electrical cycle
#define FULLSTEP  TABLELENGTH / 4   // 4 Fullsteps in an electrical cycle

#define FULLSTEP0   0 * TABLELENGTH / 8
#define FULLSTEP1   2 * TABLELENGTH / 8
#define FULLSTEP2   4 * TABLELENGTH / 8
#define FULLSTEP3   6 * TABLELENGTH / 8

#define HALFSTEP0   1 * TABLELENGTH / 8
#define HALFSTEP1   3 * TABLELENGTH / 8
#define HALFSTEP2   5 * TABLELENGTH / 8
#define HALFSTEP3   7 * TABLELENGTH / 8

#define _90degrees TABLELENGTH / 4
#define _60degrees TABLELENGTH / 6


#define NEGATIVE_TERMINAL 0
#define POSITIVE_TERMINAL 1
enum{COIL1_IN, COIL1_OUT, COIL2_IN, COIL2_OUT};

#if (MEASURE == NEGATIVE_TERMINAL)
#define COIL2_POS_CURRENT   COIL1_IN
#define COIL2_NEG_CURRENT   COIL1_OUT
#define COIL1_POS_CURRENT   COIL2_IN
#define COIL1_NEG_CURRENT   COIL2_OUT
#endif

#if (MEASURE == POSITIVE_TERMINAL)
#define COIL2_POS_CURRENT   COIL1_OUT
#define COIL2_NEG_CURRENT   COIL1_IN
#define COIL1_POS_CURRENT   COIL2_OUT
#define COIL1_NEG_CURRENT   COIL2_IN
#endif
 
#define CW   0
#define CCW  1

#define MOTOR_0_COIL2 MCDC0
#define MOTOR_0_COIL1 MCDC1

#define MOTOR_1_COIL2 MCDC2
#define MOTOR_1_COIL1 MCDC3

#define MOTOR_2_COIL2 MCDC4
#define MOTOR_2_COIL1 MCDC5

#define MOTOR_3_COIL2 MCDC6
#define MOTOR_3_COIL1 MCDC7


//------------------------------------------------------------------


//#define u8  unsigned char
//#define u16 unsigned int
//#define u32 unsigned long
//#define s8  signed char
//#define s16 signed int
//#define s32 signed long

//----------------------------------
// Debug variables - START
//----------------------------------

typedef struct
{
  u8   microstepcount;     
  u8   detectmarker;
  u8   speedmarker;
  u8   Fullstep;
  s16  FS0123;
  s16  FS0;
  s16  FS1;
  s16  FS2;
  s16  FS3;
  s16  FS0_diff;
  s16  FS1_diff;
  s16  FS2_diff;
  s16  FS3_diff;
  s16  Coil1;
  s16  Coil2;
  s16  Coil1_diff;
  s16  Coil2_diff;
  s16  TotalValue;
  s16  TotalValueZ1;
  s16  TotalValueZ2;
  s16  TotalValue_diff;
  s16  TotalValue_diff2;
  s16  filtereddiff;
  s16  filtereddiffZ1;
  s16  diffdiff;
}debugvars;

//----------------------------------
// Debug variables - END
//----------------------------------


//----------------------------------
// Motor Paramater Defines - START
//----------------------------------
typedef struct 
    {
    debugvars DEBUG;

    u8    u8RTZsample_pt0;
    u8    u8RTZsample_pt1;
    u8    u8RTZsample_pt2;
    u8    u8RTZsample_pt3;
    u8    u8RTZon;
    u8    u8RTZdetectdelay;    
    s8    u8RTZincrementer;
    u8    u8RTZcoilsample;   
    u16   u16RTZcount;
    u16   u16RTZsteptime;
    u16   u16RTZsteptimer;
    u16   u16RTZspeed;
    u16   u16RTZstartspeed;

    u8    dummy;
    u8    u8RTZflip;
    u8    u8RTZdetected;
    u8    u8RTZmicrostepsincycle;
    s16   s16RTZvalue;
    s16   s16RTZsampleZ0;
    s16   s16RTZsampleZ1;
    s16   s16RTZsampleZ2;
    s16   s16RTZsampleZ3;
    s16   s16RTZfiltered;
    s16   s16RTZfilteredZ1;
    s16   s16RTZfilteredZ2;
    s16   s16RTZdiff;
    s16   s16RTZdiff2;
    s16   s16RTZthreshold;

    u8    u8_mtr;
    u8    u8_mtrtype;
    u8    u8_pwmchan;
    u8    u8_stopdirection;
    u8    u8_zerostep;
    u8    u8_step;
    u8    u8_decayval;
    u8    u8_dir;
    u8    u8_cmddir;
    u8    u8_vscaler;
    u16   u16_vel;
    u16   u16_cmdvel;
    u16   u16_maxvel;
    u32   u32_pos;
    u32   u32_cmdpos;
    }motorparams;
//----------------------------------
// Motor Paramater Defines - END
//----------------------------------


    

//------------------------------------------------------------------
// Globals defined in this file
//------------------------------------------------------------------  
#ifdef MOTOR_ALGO_C
  #define EXT_MOTOR_ALGO
#else
  #define EXT_MOTOR_ALGO extern
#endif

EXT_MOTOR_ALGO  motorparams Motor0, Motor1, Motor2, Motor3;
EXT_MOTOR_ALGO  u16   u16SysTimer;
EXT_MOTOR_ALGO  u8    usteps_in_cycle;
EXT_MOTOR_ALGO  u8    marker, TEST60, TEST90;

enum 	{MMTmtr, DENSOmtr, VIDmtr, SWITECmtr};
enum 	{_90degType, _60degType};
enum 	{ustepRTZ, HalfstepRTZ, FullstepRTZ};
enum  {RTZdone, RTZnewstep, RTZflyback, RTZteststall};
  
//------------------------------------------------------------------
//------------------------------------------------------------------
// Constants / Tables defined in this file
//------------------------------------------------------------------

#ifdef MOTOR_ALGO_C

    #if (TABLELENGTH == 32)
    EXT_MOTOR_ALGO const u8  SIN_tbl [32]= {0,50,98,142,180,212,236,250,255,250,236,212,180,142,98,50,0,50,98,142,180,212,236,250,255,250,236,212,180,142,98,50,};
    #define TABLESCALEFACTOR  4096
    #define SHIFT       12                  // 12 - 32pt, 11 - 64 pt, 10 - 128pt, 9 - 256pt
    #endif

    #if (TABLELENGTH == 64)
    EXT_MOTOR_ALGO const u8  SIN_tbl [64]= {0,25,50,74,98,120,142,162,180,197,212,225,236,244,250,254,255,254,250,244,236,225,212,197,180,162,142,120,98,74,50,25,0,25,50,74,98,120,142,162,180,197,212,225,236,244,250,254,255,254,250,244,236,225,212,197,180,162,142,120,98,74,50,25};
    #define TABLESCALEFACTOR  2048
    #define SHIFT       11                  // 12 - 32pt, 11 - 64 pt, 10 - 128pt, 9 - 256pt
    #endif

    #if (TABLELENGTH == 128)
    EXT_MOTOR_ALGO const u8  SIN_tbl [128] = {0,13,25,37,50,62,74,86,98,109,120,131,142,152,162,171,180,189,197,205,212,219,225,231,236,240,244,247,250,252,254,255,255,255,254,252,250,247,244,240,236,231,225,219,212,205,197,189,180,171,162,152,142,131,120,109,98,86,74,62,50,37,25,13,0,13,25,37,50,62,74,86,98,109,120,131,142,152,162,171,180,189,197,205,212,219,225,231,236,240,244,247,250,252,254,255,255,255,254,252,250,247,244,240,236,231,225,219,212,205,197,189,180,171,162,152,142,131,120,109,98,86,74,62,50,37,25,13};
    #define TABLESCALEFACTOR  1024
    #define SHIFT       10                  // 12 - 32pt, 11 - 64 pt, 10 - 128pt, 9 - 256pt
    #endif

    #if (TABLELENGTH == 256)
    EXT_MOTOR_ALGO const u8  SIN_tbl [256] = {0,6,13,19,25,31,37,44,50,56,62,68,74,80,86,92,98,103,109,115,120,126,131,136,142,147,152,157,162,167,171,176,180,185,189,193,197,201,205,208,212,215,219,222,225,228,231,233,236,238,240,242,244,246,247,249,250,251,252,253,254,254,255,255,255,255,255,254,254,253,252,251,250,249,247,246,244,242,240,238,236,233,231,228,225,222,219,215,212,208,205,201,197,193,189,185,180,176,171,167,162,157,152,147,142,136,131,126,120,115,109,103,98,92,86,80,74,68,62,56,50,44,37,31,25,19,13,6,0,6,13,19,25,31,37,44,50,56,62,68,74,80,86,92,98,103,109,115,120,126,131,136,142,147,152,157,162,167,171,176,180,185,189,193,197,201,205,208,212,215,219,222,225,228,231,233,236,238,240,242,244,246,247,249,250,251,252,253,254,254,255,255,255,255,255,254,254,253,252,251,250,249,247,246,244,242,240,238,236,233,231,228,225,222,219,215,212,208,205,201,197,193,189,185,180,176,171,167,162,157,152,147,142,136,131,126,120,115,109,103,98,92,86,80,74,68,62,56,50,44,37,31,25,19,13,6};
    #define TABLESCALEFACTOR  512
    #define SHIFT        9                  // 12 - 32pt, 11 - 64 pt, 10 - 128pt, 9 - 256pt
    #endif

    // This is a 4500deg/s^2 accleration ramp assuming a 16 point sine table. Times are in Milliseconds.
    // The table assumes 2deg / electrical revelolution
    // MMT or SWITEC compatible motor
    // const uint MicroStepRTZramp [] = {7454,3087,2369,1997,1760,1591,1463,1362,1279,1210,1150,1099,1054,1014,979,947,918,891,867,844,823,804,786,769,753,738,724,711,698,686,675,664,654,644,635,626,617,609,601,593,586,579,572,565,559,553,547,541,535,530,524,519,514,510,505,500,496,491,487,483,479,475,471,468,464,460,457,454,450,447,444,441,438,435,432,429,426,423,421,418,415,413,410,408,405,403,401,398,396,394,392,390,387,385,383,381,379,377,376,374,372,370,368,366,365,363,361,359,358,356,355,353,351,350,348,347,345,344,342,341,340,338,337,335,334,333,331,330,329,327,326,325,324,323,321,320,319,318,317,316,314,313,312};

    // Denso Motor - 4000 deg/s^2 up to 400 deg/s
    //const u16 MicroStepRTZramp [] = { 9682,4011,3077,2594,2286,2066,1900,1769,1661,1571,1494,1428,1370,1318,1272,1230,1192,1157,1126,1096,1069,1044,1021,999,978,959,940,923,907,891,877,863,849,836,824,813,801,791,780,770,761,752,743,734,726,718,710,702,695,688,681,675,668,662,656,650,644,638,633,628,622,617,612,608,603,598,594,589,585,581,577,573,569,565,561,557,554,550,546,543,540,536,533,530,527,524,521,518,515,512,509,506,503,501,498,495,493,490,488,485,483,481,478,476,474,471,469};
    // Denso Motor - 2500 deg/s^2 up to 400 deg/s
    //const uint MicroStepRTZramp[] = { 12247,5073,3893,3282,2891,2614,2404,2237,2101,1987,1890,1806,1732,1667,1608,1556,1508,1464,1424,1387,1353,1321,1291,1263,1237,1213,1190,1168,1147,1128,1109,1091,1074,1058,1043,1028,1014,1000,987,974,962,951,939,928,918,908,898,889,879,870,862,853,845,837,830,822,815,808,801,794,787,781,775,768,762,757,751,745,740,735,729,724,719,714,709,705,700,696,691,687,683,678,674,670,666,662,658,655,651,647,644,640,637,633,630,627,623,620,617,614,611,608,605,602,599,596,593,591,588,585,583,580,577,575,572,570,567,565,563,560,558,556,553,551,549,547,544,542,540,538,536,534,532,530,528,526,524,522,520,518,517,515,513,511,509,508,506,504,503,501,499,498,496,494,493,491,490,488,486,485,483,482,480,479,477,476,475,473,472,470,469};
    EXT_MOTOR_ALGO const u16 MicroStepRTZramp[] = {47434,17362,11344,8453,6742,5610,4804,4201,3733,3358,3052,2798,2582,2397,2237,2101,1987,1890,1806,1732,1667,1608,1556,1508,1464,1424,1387,1353,1321,1291,1263,1237,1213,1190,1168,1147,1128,1109,1091,1074,1058,1043,1028,1014,1000,987,974,962,951,939,928,918,908,898,889,879,870,862,853,845,837,830,822,815,808,801,794,787,781,775,768,762,757,751,745,740,735,729,724,719,714,709,705,700,696,691,687,683,678,674,670,666,662,658,655,651,647,644,640,637,633,630,627,623,620,617,614,611,608,605,602,599,596,593,591,588,585,583,580,577,575,572,570,567,565,563,560,558,556,553,551,549,547,544,542,540,538,536,534,532,530,528,526,524,522,520,518,517,515,513,511,509,508,506,504,503,501,499,498,496,494,493,491,490,488,486,485,483,482,480,479,477,476,475,473,472,470,469,0};
#endif
//------------------------------------------------------------------
 

//------------------------------------------------------------------
// Function Prototypes
//------------------------------------------------------------------  
void Init_Timers(void);
void Init_Motor_Params(void);
void InitRTZ(motorparams *mtr, u16 rtzspeed, u16 rtzstartspeed, s16 threshold, u8 detectdelay);
void AlignMotorOnFullStep(motorparams *mtr);
void WaitForRTZdone(motorparams *mtr);
void WaitUntilStopped(motorparams *mtr);
u8 CheckRTZdone(motorparams *mtr);
u8 CheckIfStopped(motorparams *mtr);
void GoToPosition(motorparams *mtr, u32 position);
void GoToFullStepPosition(motorparams *mtr, u32 position);
void RTZmotor(motorparams *mtr);
void DelayMS(u16 delay);
void RTZallmotors(void);
void RTZmotor(motorparams *mtr);
//-------------------------------------------------------------------- 

//------------------------------------------------------------------




#endif